#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
# PySNMP MIB module SNMP-VIEW-BASED-ACM-MIB (https://www.pysnmp.com/pysnmp)
# ASN.1 source file://asn1/SNMP-VIEW-BASED-ACM-MIB
# Produced by pysmi-1.5.8 at Sat Nov  2 15:25:41 2024
# On host MacBook-Pro.local platform Darwin version 24.1.0 by user lextm
# Using Python version 3.12.0 (main, Nov 14 2023, 23:52:11) [Clang 15.0.0 (clang-1500.0.40.1)]
#
# IMPORTANT: customization

# Import base ASN.1 objects even if this MIB does not use it

(Integer,
 OctetString,
 ObjectIdentifier) = mibBuilder.import_symbols(
    "ASN1",
    "Integer",
    "OctetString",
    "ObjectIdentifier")

(NamedValues,) = mibBuilder.import_symbols(
    "ASN1-ENUMERATION",
    "NamedValues")
(ConstraintsIntersection,
 ConstraintsUnion,
 SingleValueConstraint,
 ValueRangeConstraint,
 ValueSizeConstraint) = mibBuilder.import_symbols(
    "ASN1-REFINEMENT",
    "ConstraintsIntersection",
    "ConstraintsUnion",
    "SingleValueConstraint",
    "ValueRangeConstraint",
    "ValueSizeConstraint")

# Import SMI symbols from the MIBs this MIB depends on

(SnmpAdminString,
 SnmpSecurityLevel,
 SnmpSecurityModel) = mibBuilder.import_symbols(
    "SNMP-FRAMEWORK-MIB",
    "SnmpAdminString",
    "SnmpSecurityLevel",
    "SnmpSecurityModel")

(ModuleCompliance,
 NotificationGroup,
 ObjectGroup) = mibBuilder.import_symbols(
    "SNMPv2-CONF",
    "ModuleCompliance",
    "NotificationGroup",
    "ObjectGroup")

(Bits,
 Counter32,
 Counter64,
 Gauge32,
 Integer32,
 IpAddress,
 ModuleIdentity,
 MibIdentifier,
 NotificationType,
 ObjectIdentity,
 MibScalar,
 MibTable,
 MibTableRow,
 MibTableColumn,
 TimeTicks,
 Unsigned32,
 iso,
 snmpModules) = mibBuilder.import_symbols(
    "SNMPv2-SMI",
    "Bits",
    "Counter32",
    "Counter64",
    "Gauge32",
    "Integer32",
    "IpAddress",
    "ModuleIdentity",
    "MibIdentifier",
    "NotificationType",
    "ObjectIdentity",
    "MibScalar",
    "MibTable",
    "MibTableRow",
    "MibTableColumn",
    "TimeTicks",
    "Unsigned32",
    "iso",
    "snmpModules")

(DisplayString,
 RowStatus,
 StorageType,
 TextualConvention,
 TestAndIncr) = mibBuilder.import_symbols(
    "SNMPv2-TC",
    "DisplayString",
    "RowStatus",
    "StorageType",
    "TextualConvention",
    "TestAndIncr")


# MODULE-IDENTITY

snmpVacmMIB = ModuleIdentity(
    (1, 3, 6, 1, 6, 3, 16)
)
if mibBuilder.loadTexts:
    snmpVacmMIB.setRevisions(
        ("2002-10-16 00:00",
         "1999-01-20 00:00",
         "1997-11-20 00:00")
    )
if mibBuilder.loadTexts:
    snmpVacmMIB.setLastUpdated("200210160000Z")
if mibBuilder.loadTexts:
    snmpVacmMIB.setOrganization("SNMPv3 Working Group")
if mibBuilder.loadTexts:
    snmpVacmMIB.setContactInfo("WG-email: snmpv3@lists.tislabs.com Subscribe: majordomo@lists.tislabs.com In message body: subscribe snmpv3 Co-Chair: Russ Mundy Network Associates Laboratories postal: 15204 Omega Drive, Suite 300 Rockville, MD 20850-4601 USA email: mundy@tislabs.com phone: +1 301-947-7107 Co-Chair: David Harrington Enterasys Networks Postal: 35 Industrial Way P. O. Box 5004 Rochester, New Hampshire 03866-5005 USA EMail: dbh@enterasys.com Phone: +1 603-337-2614 Co-editor: Bert Wijnen Lucent Technologies postal: Schagen 33 3461 GL Linschoten Netherlands email: bwijnen@lucent.com phone: +31-348-480-685 Co-editor: Randy Presuhn BMC Software, Inc. postal: 2141 North First Street San Jose, CA 95131 USA email: randy_presuhn@bmc.com phone: +1 408-546-1006 Co-editor: Keith McCloghrie Cisco Systems, Inc. postal: 170 West Tasman Drive San Jose, CA 95134-1706 USA email: kzm@cisco.com phone: +1-408-526-5260 ")
if mibBuilder.loadTexts:
    snmpVacmMIB.setDescription("The management information definitions for the View-based Access Control Model for SNMP. Copyright (C) The Internet Society (2002). This version of this MIB module is part of RFC 3415; see the RFC itself for full legal notices. ")


# Types definitions


# TEXTUAL-CONVENTIONS



# MIB Managed Objects in the order of their OIDs

_VacmMIBObjects_ObjectIdentity = ObjectIdentity
vacmMIBObjects = _VacmMIBObjects_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 16, 1)
)
_VacmContextTable_Object = MibTable
vacmContextTable = _VacmContextTable_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 1)
)
if mibBuilder.loadTexts:
    vacmContextTable.setStatus("current")
if mibBuilder.loadTexts:
    vacmContextTable.setDescription("The table of locally available contexts. This table provides information to SNMP Command Generator applications so that they can properly configure the vacmAccessTable to control access to all contexts at the SNMP entity. This table may change dynamically if the SNMP entity allows that contexts are added/deleted dynamically (for instance when its configuration changes). Such changes would happen only if the management instrumentation at that SNMP entity recognizes more (or fewer) contexts. The presence of entries in this table and of entries in the vacmAccessTable are independent. That is, a context identified by an entry in this table is not necessarily referenced by any entries in the vacmAccessTable; and the context(s) referenced by an entry in the vacmAccessTable does not necessarily currently exist and thus need not be identified by an entry in this table. This table must be made accessible via the default context so that Command Responder applications have a standard way of retrieving the information. This table is read-only. It cannot be configured via SNMP. ")
_VacmContextEntry_Object = MibTableRow
vacmContextEntry = _VacmContextEntry_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 1, 1)
)
vacmContextEntry.setIndexNames(
    (0, "SNMP-VIEW-BASED-ACM-MIB", "vacmContextName"),
)
if mibBuilder.loadTexts:
    vacmContextEntry.setStatus("current")
if mibBuilder.loadTexts:
    vacmContextEntry.setDescription("Information about a particular context.")


class _VacmContextName_Type(SnmpAdminString):
    """Custom type vacmContextName based on SnmpAdminString"""
    subtypeSpec = SnmpAdminString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(0, 32),
    )


_VacmContextName_Type.__name__ = "SnmpAdminString"
_VacmContextName_Object = MibTableColumn
vacmContextName = _VacmContextName_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 1, 1, 1),
    _VacmContextName_Type()
)
vacmContextName.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    vacmContextName.setStatus("current")
if mibBuilder.loadTexts:
    vacmContextName.setDescription("A human readable name identifying a particular context at a particular SNMP entity. The empty contextName (zero length) represents the default context. ")

# IMPORTANT: customization
# The RowStatus column is not present in the MIB
vacmContextStatus = MibTableColumn(
    (1, 3, 6, 1, 6, 3, 16, 1, 1, 1, 2), RowStatus()
).setMaxAccess("read-create")
if mibBuilder.loadTexts:
    vacmContextStatus.setStatus("current")
if mibBuilder.loadTexts:
    vacmContextStatus.setDescription(
        "The status of this conceptual row. Until instances of all corresponding columns are appropriately configured, the value of the corresponding instance of the vacmContextTableStatus column is 'notReady'. In particular, a newly created row cannot be made active until a value has been set for vacmContextName. The RowStatus TC [RFC2579] requires that this DESCRIPTION clause states under which circumstances other objects in this row can be modified: The value of this object has no effect on whether other objects in this conceptual row can be modified. "
    )

_VacmSecurityToGroupTable_Object = MibTable
vacmSecurityToGroupTable = _VacmSecurityToGroupTable_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 2)
)
if mibBuilder.loadTexts:
    vacmSecurityToGroupTable.setStatus("current")
if mibBuilder.loadTexts:
    vacmSecurityToGroupTable.setDescription("This table maps a combination of securityModel and securityName into a groupName which is used to define an access control policy for a group of principals. ")
_VacmSecurityToGroupEntry_Object = MibTableRow
vacmSecurityToGroupEntry = _VacmSecurityToGroupEntry_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 2, 1)
)
vacmSecurityToGroupEntry.setIndexNames(
    (0, "SNMP-VIEW-BASED-ACM-MIB", "vacmSecurityModel"),
    (0, "SNMP-VIEW-BASED-ACM-MIB", "vacmSecurityName"),
)
if mibBuilder.loadTexts:
    vacmSecurityToGroupEntry.setStatus("current")
if mibBuilder.loadTexts:
    vacmSecurityToGroupEntry.setDescription("An entry in this table maps the combination of a securityModel and securityName into a groupName. ")


class _VacmSecurityModel_Type(SnmpSecurityModel):
    """Custom type vacmSecurityModel based on SnmpSecurityModel"""
    subtypeSpec = SnmpSecurityModel.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueRangeConstraint(1, 2147483647),
    )


_VacmSecurityModel_Type.__name__ = "SnmpSecurityModel"
_VacmSecurityModel_Object = MibTableColumn
vacmSecurityModel = _VacmSecurityModel_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 2, 1, 1),
    _VacmSecurityModel_Type()
)
vacmSecurityModel.setMaxAccess("not-accessible")
if mibBuilder.loadTexts:
    vacmSecurityModel.setStatus("current")
if mibBuilder.loadTexts:
    vacmSecurityModel.setDescription("The Security Model, by which the vacmSecurityName referenced by this entry is provided. Note, this object may not take the 'any' (0) value. ")


class _VacmSecurityName_Type(SnmpAdminString):
    """Custom type vacmSecurityName based on SnmpAdminString"""
    subtypeSpec = SnmpAdminString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(1, 32),
    )


_VacmSecurityName_Type.__name__ = "SnmpAdminString"
_VacmSecurityName_Object = MibTableColumn
vacmSecurityName = _VacmSecurityName_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 2, 1, 2),
    _VacmSecurityName_Type()
)
vacmSecurityName.setMaxAccess("not-accessible")
if mibBuilder.loadTexts:
    vacmSecurityName.setStatus("current")
if mibBuilder.loadTexts:
    vacmSecurityName.setDescription("The securityName for the principal, represented in a Security Model independent format, which is mapped by this entry to a groupName. ")


class _VacmGroupName_Type(SnmpAdminString):
    """Custom type vacmGroupName based on SnmpAdminString"""
    subtypeSpec = SnmpAdminString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(1, 32),
    )


_VacmGroupName_Type.__name__ = "SnmpAdminString"
_VacmGroupName_Object = MibTableColumn
vacmGroupName = _VacmGroupName_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 2, 1, 3),
    _VacmGroupName_Type()
)
vacmGroupName.setMaxAccess("read-create")
if mibBuilder.loadTexts:
    vacmGroupName.setStatus("current")
if mibBuilder.loadTexts:
    vacmGroupName.setDescription("The name of the group to which this entry (e.g., the combination of securityModel and securityName) belongs. This groupName is used as index into the vacmAccessTable to select an access control policy. However, a value in this table does not imply that an instance with the value exists in table vacmAccesTable. ")


class _VacmSecurityToGroupStorageType_Type(StorageType):
    """Custom type vacmSecurityToGroupStorageType based on StorageType"""
    defaultValue = 3


_VacmSecurityToGroupStorageType_Type.__name__ = "StorageType"
_VacmSecurityToGroupStorageType_Object = MibTableColumn
vacmSecurityToGroupStorageType = _VacmSecurityToGroupStorageType_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 2, 1, 4),
    _VacmSecurityToGroupStorageType_Type()
)
vacmSecurityToGroupStorageType.setMaxAccess("read-create")
if mibBuilder.loadTexts:
    vacmSecurityToGroupStorageType.setStatus("current")
if mibBuilder.loadTexts:
    vacmSecurityToGroupStorageType.setDescription("The storage type for this conceptual row. Conceptual rows having the value 'permanent' need not allow write-access to any columnar objects in the row. ")
_VacmSecurityToGroupStatus_Type = RowStatus
_VacmSecurityToGroupStatus_Object = MibTableColumn
vacmSecurityToGroupStatus = _VacmSecurityToGroupStatus_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 2, 1, 5),
    _VacmSecurityToGroupStatus_Type()
)
vacmSecurityToGroupStatus.setMaxAccess("read-create")
if mibBuilder.loadTexts:
    vacmSecurityToGroupStatus.setStatus("current")
if mibBuilder.loadTexts:
    vacmSecurityToGroupStatus.setDescription("The status of this conceptual row. Until instances of all corresponding columns are appropriately configured, the value of the corresponding instance of the vacmSecurityToGroupStatus column is 'notReady'. In particular, a newly created row cannot be made active until a value has been set for vacmGroupName. The RowStatus TC [RFC2579] requires that this DESCRIPTION clause states under which circumstances other objects in this row can be modified: The value of this object has no effect on whether other objects in this conceptual row can be modified. ")
_VacmAccessTable_Object = MibTable
vacmAccessTable = _VacmAccessTable_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 4)
)
if mibBuilder.loadTexts:
    vacmAccessTable.setStatus("current")
if mibBuilder.loadTexts:
    vacmAccessTable.setDescription("The table of access rights for groups. Each entry is indexed by a groupName, a contextPrefix, a securityModel and a securityLevel. To determine whether access is allowed, one entry from this table needs to be selected and the proper viewName from that entry must be used for access control checking. To select the proper entry, follow these steps: 1) the set of possible matches is formed by the intersection of the following sets of entries: the set of entries with identical vacmGroupName the union of these two sets: - the set with identical vacmAccessContextPrefix - the set of entries with vacmAccessContextMatch value of 'prefix' and matching vacmAccessContextPrefix intersected with the union of these two sets: - the set of entries with identical vacmSecurityModel - the set of entries with vacmSecurityModel value of 'any' intersected with the set of entries with vacmAccessSecurityLevel value less than or equal to the requested securityLevel 2) if this set has only one member, we're done otherwise, it comes down to deciding how to weight the preferences between ContextPrefixes, SecurityModels, and SecurityLevels as follows: a) if the subset of entries with securityModel matching the securityModel in the message is not empty, then discard the rest. b) if the subset of entries with vacmAccessContextPrefix matching the contextName in the message is not empty, then discard the rest c) discard all entries with ContextPrefixes shorter than the longest one remaining in the set d) select the entry with the highest securityLevel Please note that for securityLevel noAuthNoPriv, all groups are really equivalent since the assumption that the securityName has been authenticated does not hold. ")
_VacmAccessEntry_Object = MibTableRow
vacmAccessEntry = _VacmAccessEntry_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 4, 1)
)
vacmAccessEntry.setIndexNames(
    (0, "SNMP-VIEW-BASED-ACM-MIB", "vacmGroupName"),
    (0, "SNMP-VIEW-BASED-ACM-MIB", "vacmAccessContextPrefix"),
    (0, "SNMP-VIEW-BASED-ACM-MIB", "vacmAccessSecurityModel"),
    (0, "SNMP-VIEW-BASED-ACM-MIB", "vacmAccessSecurityLevel"),
)
if mibBuilder.loadTexts:
    vacmAccessEntry.setStatus("current")
if mibBuilder.loadTexts:
    vacmAccessEntry.setDescription("An access right configured in the Local Configuration Datastore (LCD) authorizing access to an SNMP context. Entries in this table can use an instance value for object vacmGroupName even if no entry in table vacmAccessSecurityToGroupTable has a corresponding value for object vacmGroupName. ")


class _VacmAccessContextPrefix_Type(SnmpAdminString):
    """Custom type vacmAccessContextPrefix based on SnmpAdminString"""
    subtypeSpec = SnmpAdminString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(0, 32),
    )


_VacmAccessContextPrefix_Type.__name__ = "SnmpAdminString"
_VacmAccessContextPrefix_Object = MibTableColumn
vacmAccessContextPrefix = _VacmAccessContextPrefix_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 4, 1, 1),
    _VacmAccessContextPrefix_Type()
)
vacmAccessContextPrefix.setMaxAccess("not-accessible")
if mibBuilder.loadTexts:
    vacmAccessContextPrefix.setStatus("current")
if mibBuilder.loadTexts:
    vacmAccessContextPrefix.setDescription("In order to gain the access rights allowed by this conceptual row, a contextName must match exactly (if the value of vacmAccessContextMatch is 'exact') or partially (if the value of vacmAccessContextMatch is 'prefix') to the value of the instance of this object. ")
_VacmAccessSecurityModel_Type = SnmpSecurityModel
_VacmAccessSecurityModel_Object = MibTableColumn
vacmAccessSecurityModel = _VacmAccessSecurityModel_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 4, 1, 2),
    _VacmAccessSecurityModel_Type()
)
vacmAccessSecurityModel.setMaxAccess("not-accessible")
if mibBuilder.loadTexts:
    vacmAccessSecurityModel.setStatus("current")
if mibBuilder.loadTexts:
    vacmAccessSecurityModel.setDescription("In order to gain the access rights allowed by this conceptual row, this securityModel must be in use. ")
_VacmAccessSecurityLevel_Type = SnmpSecurityLevel
_VacmAccessSecurityLevel_Object = MibTableColumn
vacmAccessSecurityLevel = _VacmAccessSecurityLevel_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 4, 1, 3),
    _VacmAccessSecurityLevel_Type()
)
vacmAccessSecurityLevel.setMaxAccess("not-accessible")
if mibBuilder.loadTexts:
    vacmAccessSecurityLevel.setStatus("current")
if mibBuilder.loadTexts:
    vacmAccessSecurityLevel.setDescription("The minimum level of security required in order to gain the access rights allowed by this conceptual row. A securityLevel of noAuthNoPriv is less than authNoPriv which in turn is less than authPriv. If multiple entries are equally indexed except for this vacmAccessSecurityLevel index, then the entry which has the highest value for vacmAccessSecurityLevel is selected. ")


class _VacmAccessContextMatch_Type(Integer32):
    """Custom type vacmAccessContextMatch based on Integer32"""
    defaultValue = 1

    subtypeSpec = Integer32.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        SingleValueConstraint(
            *(1,
              2)
        )
    )
    namedValues = NamedValues(
        *(("exact", 1),
          ("prefix", 2))
    )


_VacmAccessContextMatch_Type.__name__ = "Integer32"
_VacmAccessContextMatch_Object = MibTableColumn
vacmAccessContextMatch = _VacmAccessContextMatch_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 4, 1, 4),
    _VacmAccessContextMatch_Type()
)
vacmAccessContextMatch.setMaxAccess("read-create")
if mibBuilder.loadTexts:
    vacmAccessContextMatch.setStatus("current")
if mibBuilder.loadTexts:
    vacmAccessContextMatch.setDescription("If the value of this object is exact(1), then all rows where the contextName exactly matches vacmAccessContextPrefix are selected. If the value of this object is prefix(2), then all rows where the contextName whose starting octets exactly match vacmAccessContextPrefix are selected. This allows for a simple form of wildcarding. ")


class _VacmAccessReadViewName_Type(SnmpAdminString):
    """Custom type vacmAccessReadViewName based on SnmpAdminString"""
    defaultHexValue = ""

    subtypeSpec = SnmpAdminString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(0, 32),
    )


_VacmAccessReadViewName_Type.__name__ = "SnmpAdminString"
_VacmAccessReadViewName_Object = MibTableColumn
vacmAccessReadViewName = _VacmAccessReadViewName_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 4, 1, 5),
    _VacmAccessReadViewName_Type()
)
vacmAccessReadViewName.setMaxAccess("read-create")
if mibBuilder.loadTexts:
    vacmAccessReadViewName.setStatus("current")
if mibBuilder.loadTexts:
    vacmAccessReadViewName.setDescription("The value of an instance of this object identifies the MIB view of the SNMP context to which this conceptual row authorizes read access. The identified MIB view is that one for which the vacmViewTreeFamilyViewName has the same value as the instance of this object; if the value is the empty string or if there is no active MIB view having this value of vacmViewTreeFamilyViewName, then no access is granted. ")


class _VacmAccessWriteViewName_Type(SnmpAdminString):
    """Custom type vacmAccessWriteViewName based on SnmpAdminString"""
    defaultHexValue = ""

    subtypeSpec = SnmpAdminString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(0, 32),
    )


_VacmAccessWriteViewName_Type.__name__ = "SnmpAdminString"
_VacmAccessWriteViewName_Object = MibTableColumn
vacmAccessWriteViewName = _VacmAccessWriteViewName_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 4, 1, 6),
    _VacmAccessWriteViewName_Type()
)
vacmAccessWriteViewName.setMaxAccess("read-create")
if mibBuilder.loadTexts:
    vacmAccessWriteViewName.setStatus("current")
if mibBuilder.loadTexts:
    vacmAccessWriteViewName.setDescription("The value of an instance of this object identifies the MIB view of the SNMP context to which this conceptual row authorizes write access. The identified MIB view is that one for which the vacmViewTreeFamilyViewName has the same value as the instance of this object; if the value is the empty string or if there is no active MIB view having this value of vacmViewTreeFamilyViewName, then no access is granted. ")


class _VacmAccessNotifyViewName_Type(SnmpAdminString):
    """Custom type vacmAccessNotifyViewName based on SnmpAdminString"""
    defaultHexValue = ""

    subtypeSpec = SnmpAdminString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(0, 32),
    )


_VacmAccessNotifyViewName_Type.__name__ = "SnmpAdminString"
_VacmAccessNotifyViewName_Object = MibTableColumn
vacmAccessNotifyViewName = _VacmAccessNotifyViewName_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 4, 1, 7),
    _VacmAccessNotifyViewName_Type()
)
vacmAccessNotifyViewName.setMaxAccess("read-create")
if mibBuilder.loadTexts:
    vacmAccessNotifyViewName.setStatus("current")
if mibBuilder.loadTexts:
    vacmAccessNotifyViewName.setDescription("The value of an instance of this object identifies the MIB view of the SNMP context to which this conceptual row authorizes access for notifications. The identified MIB view is that one for which the vacmViewTreeFamilyViewName has the same value as the instance of this object; if the value is the empty string or if there is no active MIB view having this value of vacmViewTreeFamilyViewName, then no access is granted. ")


class _VacmAccessStorageType_Type(StorageType):
    """Custom type vacmAccessStorageType based on StorageType"""
    defaultValue = 3


_VacmAccessStorageType_Type.__name__ = "StorageType"
_VacmAccessStorageType_Object = MibTableColumn
vacmAccessStorageType = _VacmAccessStorageType_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 4, 1, 8),
    _VacmAccessStorageType_Type()
)
vacmAccessStorageType.setMaxAccess("read-create")
if mibBuilder.loadTexts:
    vacmAccessStorageType.setStatus("current")
if mibBuilder.loadTexts:
    vacmAccessStorageType.setDescription("The storage type for this conceptual row. Conceptual rows having the value 'permanent' need not allow write-access to any columnar objects in the row. ")
_VacmAccessStatus_Type = RowStatus
_VacmAccessStatus_Object = MibTableColumn
vacmAccessStatus = _VacmAccessStatus_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 4, 1, 9),
    _VacmAccessStatus_Type()
)
vacmAccessStatus.setMaxAccess("read-create")
if mibBuilder.loadTexts:
    vacmAccessStatus.setStatus("current")
if mibBuilder.loadTexts:
    vacmAccessStatus.setDescription("The status of this conceptual row. The RowStatus TC [RFC2579] requires that this DESCRIPTION clause states under which circumstances other objects in this row can be modified: The value of this object has no effect on whether other objects in this conceptual row can be modified. ")
_VacmMIBViews_ObjectIdentity = ObjectIdentity
vacmMIBViews = _VacmMIBViews_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 16, 1, 5)
)
_VacmViewSpinLock_Type = TestAndIncr
_VacmViewSpinLock_Object = MibScalar
vacmViewSpinLock = _VacmViewSpinLock_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 5, 1),
    _VacmViewSpinLock_Type()
)
vacmViewSpinLock.setMaxAccess("read-write")
if mibBuilder.loadTexts:
    vacmViewSpinLock.setStatus("current")
if mibBuilder.loadTexts:
    vacmViewSpinLock.setDescription("An advisory lock used to allow cooperating SNMP Command Generator applications to coordinate their use of the Set operation in creating or modifying views. When creating a new view or altering an existing view, it is important to understand the potential interactions with other uses of the view. The vacmViewSpinLock should be retrieved. The name of the view to be created should be determined to be unique by the SNMP Command Generator application by consulting the vacmViewTreeFamilyTable. Finally, the named view may be created (Set), including the advisory lock. If another SNMP Command Generator application has altered the views in the meantime, then the spin lock's value will have changed, and so this creation will fail because it will specify the wrong value for the spin lock. Since this is an advisory lock, the use of this lock is not enforced. ")
_VacmViewTreeFamilyTable_Object = MibTable
vacmViewTreeFamilyTable = _VacmViewTreeFamilyTable_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 5, 2)
)
if mibBuilder.loadTexts:
    vacmViewTreeFamilyTable.setStatus("current")
if mibBuilder.loadTexts:
    vacmViewTreeFamilyTable.setDescription("Locally held information about families of subtrees within MIB views. Each MIB view is defined by two sets of view subtrees: - the included view subtrees, and - the excluded view subtrees. Every such view subtree, both the included and the excluded ones, is defined in this table. To determine if a particular object instance is in a particular MIB view, compare the object instance's OBJECT IDENTIFIER with each of the MIB view's active entries in this table. If none match, then the object instance is not in the MIB view. If one or more match, then the object instance is included in, or excluded from, the MIB view according to the value of vacmViewTreeFamilyType in the entry whose value of vacmViewTreeFamilySubtree has the most sub-identifiers. If multiple entries match and have the same number of sub-identifiers (when wildcarding is specified with the value of vacmViewTreeFamilyMask), then the lexicographically greatest instance of vacmViewTreeFamilyType determines the inclusion or exclusion. An object instance's OBJECT IDENTIFIER X matches an active entry in this table when the number of sub-identifiers in X is at least as many as in the value of vacmViewTreeFamilySubtree for the entry, and each sub-identifier in the value of vacmViewTreeFamilySubtree matches its corresponding sub-identifier in X. Two sub-identifiers match either if the corresponding bit of the value of vacmViewTreeFamilyMask for the entry is zero (the 'wild card' value), or if they are equal. A 'family' of subtrees is the set of subtrees defined by a particular combination of values of vacmViewTreeFamilySubtree and vacmViewTreeFamilyMask. In the case where no 'wild card' is defined in the vacmViewTreeFamilyMask, the family of subtrees reduces to a single subtree. When creating or changing MIB views, an SNMP Command Generator application should utilize the vacmViewSpinLock to try to avoid collisions. See DESCRIPTION clause of vacmViewSpinLock. When creating MIB views, it is strongly advised that first the 'excluded' vacmViewTreeFamilyEntries are created and then the 'included' entries. When deleting MIB views, it is strongly advised that first the 'included' vacmViewTreeFamilyEntries are deleted and then the 'excluded' entries. If a create for an entry for instance-level access control is received and the implementation does not support instance-level granularity, then an inconsistentName error must be returned. ")
_VacmViewTreeFamilyEntry_Object = MibTableRow
vacmViewTreeFamilyEntry = _VacmViewTreeFamilyEntry_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 5, 2, 1)
)
vacmViewTreeFamilyEntry.setIndexNames(
    (0, "SNMP-VIEW-BASED-ACM-MIB", "vacmViewTreeFamilyViewName"),
    (0, "SNMP-VIEW-BASED-ACM-MIB", "vacmViewTreeFamilySubtree"),
)
if mibBuilder.loadTexts:
    vacmViewTreeFamilyEntry.setStatus("current")
if mibBuilder.loadTexts:
    vacmViewTreeFamilyEntry.setDescription("Information on a particular family of view subtrees included in or excluded from a particular SNMP context's MIB view. Implementations must not restrict the number of families of view subtrees for a given MIB view, except as dictated by resource constraints on the overall number of entries in the vacmViewTreeFamilyTable. If no conceptual rows exist in this table for a given MIB view (viewName), that view may be thought of as consisting of the empty set of view subtrees. ")


class _VacmViewTreeFamilyViewName_Type(SnmpAdminString):
    """Custom type vacmViewTreeFamilyViewName based on SnmpAdminString"""
    subtypeSpec = SnmpAdminString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(1, 32),
    )


_VacmViewTreeFamilyViewName_Type.__name__ = "SnmpAdminString"
_VacmViewTreeFamilyViewName_Object = MibTableColumn
vacmViewTreeFamilyViewName = _VacmViewTreeFamilyViewName_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 5, 2, 1, 1),
    _VacmViewTreeFamilyViewName_Type()
)
vacmViewTreeFamilyViewName.setMaxAccess("not-accessible")
if mibBuilder.loadTexts:
    vacmViewTreeFamilyViewName.setStatus("current")
if mibBuilder.loadTexts:
    vacmViewTreeFamilyViewName.setDescription("The human readable name for a family of view subtrees. ")
_VacmViewTreeFamilySubtree_Type = ObjectIdentifier
_VacmViewTreeFamilySubtree_Object = MibTableColumn
vacmViewTreeFamilySubtree = _VacmViewTreeFamilySubtree_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 5, 2, 1, 2),
    _VacmViewTreeFamilySubtree_Type()
)
vacmViewTreeFamilySubtree.setMaxAccess("not-accessible")
if mibBuilder.loadTexts:
    vacmViewTreeFamilySubtree.setStatus("current")
if mibBuilder.loadTexts:
    vacmViewTreeFamilySubtree.setDescription("The MIB subtree which when combined with the corresponding instance of vacmViewTreeFamilyMask defines a family of view subtrees. ")


class _VacmViewTreeFamilyMask_Type(OctetString):
    """Custom type vacmViewTreeFamilyMask based on OctetString"""
    defaultHexValue = ""

    subtypeSpec = OctetString.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        ValueSizeConstraint(0, 16),
    )


_VacmViewTreeFamilyMask_Type.__name__ = "OctetString"
_VacmViewTreeFamilyMask_Object = MibTableColumn
vacmViewTreeFamilyMask = _VacmViewTreeFamilyMask_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 5, 2, 1, 3),
    _VacmViewTreeFamilyMask_Type()
)
vacmViewTreeFamilyMask.setMaxAccess("read-create")
if mibBuilder.loadTexts:
    vacmViewTreeFamilyMask.setStatus("current")
if mibBuilder.loadTexts:
    vacmViewTreeFamilyMask.setDescription("The bit mask which, in combination with the corresponding instance of vacmViewTreeFamilySubtree, defines a family of view subtrees. Each bit of this bit mask corresponds to a sub-identifier of vacmViewTreeFamilySubtree, with the most significant bit of the i-th octet of this octet string value (extended if necessary, see below) corresponding to the (8*i - 7)-th sub-identifier, and the least significant bit of the i-th octet of this octet string corresponding to the (8*i)-th sub-identifier, where i is in the range 1 through 16. Each bit of this bit mask specifies whether or not the corresponding sub-identifiers must match when determining if an OBJECT IDENTIFIER is in this family of view subtrees; a '1' indicates that an exact match must occur; a '0' indicates 'wild card', i.e., any sub-identifier value matches. Thus, the OBJECT IDENTIFIER X of an object instance is contained in a family of view subtrees if, for each sub-identifier of the value of vacmViewTreeFamilySubtree, either: the i-th bit of vacmViewTreeFamilyMask is 0, or the i-th sub-identifier of X is equal to the i-th sub-identifier of the value of vacmViewTreeFamilySubtree. If the value of this bit mask is M bits long and there are more than M sub-identifiers in the corresponding instance of vacmViewTreeFamilySubtree, then the bit mask is extended with 1's to be the required length. Note that when the value of this object is the zero-length string, this extension rule results in a mask of all-1's being used (i.e., no 'wild card'), and the family of view subtrees is the one view subtree uniquely identified by the corresponding instance of vacmViewTreeFamilySubtree. Note that masks of length greater than zero length do not need to be supported. In this case this object is made read-only. ")


class _VacmViewTreeFamilyType_Type(Integer32):
    """Custom type vacmViewTreeFamilyType based on Integer32"""
    defaultValue = 1

    subtypeSpec = Integer32.subtypeSpec
    subtypeSpec += ConstraintsUnion(
        SingleValueConstraint(
            *(1,
              2)
        )
    )
    namedValues = NamedValues(
        *(("included", 1),
          ("excluded", 2))
    )


_VacmViewTreeFamilyType_Type.__name__ = "Integer32"
_VacmViewTreeFamilyType_Object = MibTableColumn
vacmViewTreeFamilyType = _VacmViewTreeFamilyType_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 5, 2, 1, 4),
    _VacmViewTreeFamilyType_Type()
)
vacmViewTreeFamilyType.setMaxAccess("read-create")
if mibBuilder.loadTexts:
    vacmViewTreeFamilyType.setStatus("current")
if mibBuilder.loadTexts:
    vacmViewTreeFamilyType.setDescription("Indicates whether the corresponding instances of vacmViewTreeFamilySubtree and vacmViewTreeFamilyMask define a family of view subtrees which is included in or excluded from the MIB view. ")


class _VacmViewTreeFamilyStorageType_Type(StorageType):
    """Custom type vacmViewTreeFamilyStorageType based on StorageType"""
    defaultValue = 3


_VacmViewTreeFamilyStorageType_Type.__name__ = "StorageType"
_VacmViewTreeFamilyStorageType_Object = MibTableColumn
vacmViewTreeFamilyStorageType = _VacmViewTreeFamilyStorageType_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 5, 2, 1, 5),
    _VacmViewTreeFamilyStorageType_Type()
)
vacmViewTreeFamilyStorageType.setMaxAccess("read-create")
if mibBuilder.loadTexts:
    vacmViewTreeFamilyStorageType.setStatus("current")
if mibBuilder.loadTexts:
    vacmViewTreeFamilyStorageType.setDescription("The storage type for this conceptual row. Conceptual rows having the value 'permanent' need not allow write-access to any columnar objects in the row. ")
_VacmViewTreeFamilyStatus_Type = RowStatus
_VacmViewTreeFamilyStatus_Object = MibTableColumn
vacmViewTreeFamilyStatus = _VacmViewTreeFamilyStatus_Object(
    (1, 3, 6, 1, 6, 3, 16, 1, 5, 2, 1, 6),
    _VacmViewTreeFamilyStatus_Type()
)
vacmViewTreeFamilyStatus.setMaxAccess("read-create")
if mibBuilder.loadTexts:
    vacmViewTreeFamilyStatus.setStatus("current")
if mibBuilder.loadTexts:
    vacmViewTreeFamilyStatus.setDescription("The status of this conceptual row. The RowStatus TC [RFC2579] requires that this DESCRIPTION clause states under which circumstances other objects in this row can be modified: The value of this object has no effect on whether other objects in this conceptual row can be modified. ")
_VacmMIBConformance_ObjectIdentity = ObjectIdentity
vacmMIBConformance = _VacmMIBConformance_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 16, 2)
)
_VacmMIBCompliances_ObjectIdentity = ObjectIdentity
vacmMIBCompliances = _VacmMIBCompliances_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 16, 2, 1)
)
_VacmMIBGroups_ObjectIdentity = ObjectIdentity
vacmMIBGroups = _VacmMIBGroups_ObjectIdentity(
    (1, 3, 6, 1, 6, 3, 16, 2, 2)
)

# Managed Objects groups

vacmBasicGroup = ObjectGroup(
    (1, 3, 6, 1, 6, 3, 16, 2, 2, 1)
)
vacmBasicGroup.setObjects(
      *(("SNMP-VIEW-BASED-ACM-MIB", "vacmContextName"),
        ("SNMP-VIEW-BASED-ACM-MIB", "vacmGroupName"),
        ("SNMP-VIEW-BASED-ACM-MIB", "vacmSecurityToGroupStorageType"),
        ("SNMP-VIEW-BASED-ACM-MIB", "vacmSecurityToGroupStatus"),
        ("SNMP-VIEW-BASED-ACM-MIB", "vacmAccessContextMatch"),
        ("SNMP-VIEW-BASED-ACM-MIB", "vacmAccessReadViewName"),
        ("SNMP-VIEW-BASED-ACM-MIB", "vacmAccessWriteViewName"),
        ("SNMP-VIEW-BASED-ACM-MIB", "vacmAccessNotifyViewName"),
        ("SNMP-VIEW-BASED-ACM-MIB", "vacmAccessStorageType"),
        ("SNMP-VIEW-BASED-ACM-MIB", "vacmAccessStatus"),
        ("SNMP-VIEW-BASED-ACM-MIB", "vacmViewSpinLock"),
        ("SNMP-VIEW-BASED-ACM-MIB", "vacmViewTreeFamilyMask"),
        ("SNMP-VIEW-BASED-ACM-MIB", "vacmViewTreeFamilyType"),
        ("SNMP-VIEW-BASED-ACM-MIB", "vacmViewTreeFamilyStorageType"),
        ("SNMP-VIEW-BASED-ACM-MIB", "vacmViewTreeFamilyStatus"))
)
if mibBuilder.loadTexts:
    vacmBasicGroup.setStatus("current")
if mibBuilder.loadTexts:
    vacmBasicGroup.setDescription("A collection of objects providing for remote configuration of an SNMP engine which implements the SNMP View-based Access Control Model. ")


# Notification objects


# Notifications groups


# Agent capabilities


# Module compliance

vacmMIBCompliance = ModuleCompliance(
    (1, 3, 6, 1, 6, 3, 16, 2, 1, 1)
)
vacmMIBCompliance.setObjects(
    ("SNMP-VIEW-BASED-ACM-MIB", "vacmBasicGroup")
)
if mibBuilder.loadTexts:
    vacmMIBCompliance.setStatus(
        "current"
    )
if mibBuilder.loadTexts:
    vacmMIBCompliance.setDescription("The compliance statement for SNMP engines which implement the SNMP View-based Access Control Model configuration MIB. ")


# Export all MIB objects to the MIB builder

mibBuilder.export_symbols(
    "SNMP-VIEW-BASED-ACM-MIB",
    **{"snmpVacmMIB": snmpVacmMIB,
       "vacmMIBObjects": vacmMIBObjects,
       "vacmContextTable": vacmContextTable,
       "vacmContextEntry": vacmContextEntry,
       "vacmContextName": vacmContextName,
       "vacmContextStatus": vacmContextStatus,
       "vacmSecurityToGroupTable": vacmSecurityToGroupTable,
       "vacmSecurityToGroupEntry": vacmSecurityToGroupEntry,
       "vacmSecurityModel": vacmSecurityModel,
       "vacmSecurityName": vacmSecurityName,
       "vacmGroupName": vacmGroupName,
       "vacmSecurityToGroupStorageType": vacmSecurityToGroupStorageType,
       "vacmSecurityToGroupStatus": vacmSecurityToGroupStatus,
       "vacmAccessTable": vacmAccessTable,
       "vacmAccessEntry": vacmAccessEntry,
       "vacmAccessContextPrefix": vacmAccessContextPrefix,
       "vacmAccessSecurityModel": vacmAccessSecurityModel,
       "vacmAccessSecurityLevel": vacmAccessSecurityLevel,
       "vacmAccessContextMatch": vacmAccessContextMatch,
       "vacmAccessReadViewName": vacmAccessReadViewName,
       "vacmAccessWriteViewName": vacmAccessWriteViewName,
       "vacmAccessNotifyViewName": vacmAccessNotifyViewName,
       "vacmAccessStorageType": vacmAccessStorageType,
       "vacmAccessStatus": vacmAccessStatus,
       "vacmMIBViews": vacmMIBViews,
       "vacmViewSpinLock": vacmViewSpinLock,
       "vacmViewTreeFamilyTable": vacmViewTreeFamilyTable,
       "vacmViewTreeFamilyEntry": vacmViewTreeFamilyEntry,
       "vacmViewTreeFamilyViewName": vacmViewTreeFamilyViewName,
       "vacmViewTreeFamilySubtree": vacmViewTreeFamilySubtree,
       "vacmViewTreeFamilyMask": vacmViewTreeFamilyMask,
       "vacmViewTreeFamilyType": vacmViewTreeFamilyType,
       "vacmViewTreeFamilyStorageType": vacmViewTreeFamilyStorageType,
       "vacmViewTreeFamilyStatus": vacmViewTreeFamilyStatus,
       "vacmMIBConformance": vacmMIBConformance,
       "vacmMIBCompliances": vacmMIBCompliances,
       "vacmMIBCompliance": vacmMIBCompliance,
       "vacmMIBGroups": vacmMIBGroups,
       "vacmBasicGroup": vacmBasicGroup,
       "PYSNMP_MODULE_ID": snmpVacmMIB}
)



from pysnmp.smi import builder, view, compiler, rfc1902


def test_configure_mib_viewer_and_resolve_pdu_varbinds():
    # Assemble MIB browser
    mibBuilder = builder.MibBuilder()
    mibViewController = view.MibViewController(mibBuilder)
    compiler.add_mib_compiler(
        mibBuilder,
        sources=["file:///usr/share/snmp/mibs", "https://mibs.pysnmp.com/asn1/@mib@"],
    )

    # Pre-load MIB modules we expect to work with
    mibBuilder.load_modules("SNMPv2-MIB", "SNMP-COMMUNITY-MIB")

    # Check that the MIB browser is correctly configured
    assert isinstance(mibBuilder, builder.MibBuilder)
    assert isinstance(mibViewController, view.MibViewController)

    # Check that the expected MIB modules are loaded
    assert mibBuilder.load_modules("SNMPv2-MIB") is not None
    assert mibBuilder.load_modules("SNMP-COMMUNITY-MIB") is not None

    # Define expected var-binds
    expected_varbinds = [
        "SNMP-COMMUNITY-MIB::snmpTrapCommunity.0 = ",
        "SNMPv2-MIB::snmpTrapEnterprise.0 = SNMPv2-SMI::enterprises.20408.4.1.1.2",
        "SNMPv2-MIB::sysDescr.0 = my system",
    ]

    # This is what we can get in TRAP PDU
    varBinds = [
        ("1.3.6.1.6.3.18.1.4.0", ""),
        ("1.3.6.1.6.3.1.1.4.3.0", "1.3.6.1.4.1.20408.4.1.1.2"),
        ("1.3.6.1.2.1.1.1.0", "my system"),
    ]

    # Run var-binds through MIB resolver
    # You may want to catch and ignore resolution errors here
    varBinds = [
        rfc1902.ObjectType(rfc1902.ObjectIdentity(x[0]), x[1]).resolve_with_mib(
            mibViewController
        )
        for x in varBinds
    ]

    # Check that var-binds were resolved correctly
    for i, varBind in enumerate(varBinds):
        assert isinstance(varBind, rfc1902.ObjectType)
        assert varBind.prettyPrint() == expected_varbinds[i]


def test_configure_mib_viewer_and_resolve_pdu_varbinds_full():
    # Assemble MIB browser
    mibBuilder = builder.MibBuilder()
    mibViewController = view.MibViewController(mibBuilder)
    compiler.add_mib_compiler(
        mibBuilder,
        sources=["file:///usr/share/snmp/mibs", "https://mibs.pysnmp.com/asn1/@mib@"],
    )

    # Pre-load MIB modules we expect to work with
    mibBuilder.load_modules("SNMPv2-MIB", "SNMP-COMMUNITY-MIB", "PYSNMP-MIB")

    # Check that the MIB browser is correctly configured
    assert isinstance(mibBuilder, builder.MibBuilder)
    assert isinstance(mibViewController, view.MibViewController)

    # Check that the expected MIB modules are loaded
    assert mibBuilder.load_modules("SNMPv2-MIB") is not None
    assert mibBuilder.load_modules("SNMP-COMMUNITY-MIB") is not None
    assert mibBuilder.load_modules("PYSNMP-MIB") is not None

    # Define expected var-binds
    expected_varbinds = [
        "SNMP-COMMUNITY-MIB::snmpTrapCommunity.0 = ",
        "SNMPv2-MIB::snmpTrapEnterprise.0 = PYSNMP-MIB::pysnmpNotificationObjects.1.2",
        "SNMPv2-MIB::sysDescr.0 = my system",
    ]

    # This is what we can get in TRAP PDU
    varBinds = [
        ("1.3.6.1.6.3.18.1.4.0", ""),
        ("1.3.6.1.6.3.1.1.4.3.0", "1.3.6.1.4.1.20408.4.1.1.2"),
        ("1.3.6.1.2.1.1.1.0", "my system"),
    ]

    # Run var-binds through MIB resolver
    # You may want to catch and ignore resolution errors here
    varBinds = [
        rfc1902.ObjectType(rfc1902.ObjectIdentity(x[0]), x[1]).resolve_with_mib(
            mibViewController
        )
        for x in varBinds
    ]

    # Check that var-binds were resolved correctly
    for i, varBind in enumerate(varBinds):
        assert isinstance(varBind, rfc1902.ObjectType)
        assert varBind.prettyPrint() == expected_varbinds[i]


def test_syntax_integer():
    # Create MIB builder
    mib_builder = builder.MibBuilder()

    # Load MIB modules
    compiler.addMibCompiler(mib_builder)
    mib_builder.loadModules(
        "LEXTUDIO-TEST-MIB"
    )  # IMPORTANT: compile LEXTUDIO-TEST-MIB.mib in advance.

    # Get MIB object
    (mib_object,) = mib_builder.import_symbols("LEXTUDIO-TEST-MIB", "testScaledInteger")

    # Print MIB object syntax
    assert (1, 3, 6, 1, 4, 1, 60069, 9, 7) == mib_object.getName()
    assert "ScaledInteger" == mib_object.syntax.__class__.__name__


def test_syntax_unsigned():
    # Create MIB builder
    mib_builder = builder.MibBuilder()

    # Load MIB modules
    compiler.addMibCompiler(mib_builder)
    mib_builder.loadModules("LEXTUDIO-TEST-MIB")

    # Get MIB object
    (mib_object,) = mib_builder.import_symbols(
        "LEXTUDIO-TEST-MIB", "testScaledUnsigned"
    )

    # Print MIB object syntax
    assert (1, 3, 6, 1, 4, 1, 60069, 9, 8) == mib_object.getName()
    assert "ScaledUnsigned" == mib_object.syntax.__class__.__name__


def test_syntax_fixed_length_octet_string():
    # Create MIB builder
    mib_builder = builder.MibBuilder()

    # Load MIB modules
    compiler.addMibCompiler(mib_builder)
    mib_builder.loadModules(
        "LEXTUDIO-TEST-MIB"
    )  # IMPORTANT: compile LEXTUDIO-TEST-MIB.mib in advance.

    # Get MIB object
    (mib_object,) = mib_builder.import_symbols("LEXTUDIO-TEST-MIB", "testMacAddress")

    # Print MIB object syntax
    assert (1, 3, 6, 1, 4, 1, 60069, 9, 9) == mib_object.getName()
    assert "MacAddress" == mib_object.syntax.__class__.__name__



import pytest
from unittest.mock import patch
from pysnmp.smi import builder, view, rfc1902, error


def test_add_asn1_mib_source():
    mibBuilder = builder.MibBuilder()
    mibView = view.MibViewController(mibBuilder)
    mibVar = rfc1902.ObjectIdentity("IF-MIB", "ifInOctets", 1)
    mibVar.add_asn1_mib_source("https://mibs.pysnmp.com/asn1/@mib@")
    identity = mibVar.resolve_with_mib(mibView)
    assert mibVar.prettyPrint() == "IF-MIB::ifInOctets.1"

    module, symbol, name = identity.get_mib_symbol()
    assert symbol == "ifInOctets"

    next_module = mibView.get_next_module_name("IF-MIB")
    assert next_module == "SNMPv2-CONF"

    oid, label, suffix = mibView.get_first_node_name()
    oid, label, suffix = mibView.get_next_node_name(oid)
    assert oid == (0, 0)
    assert label == ("itu-t", "zeroDotZero")



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
(MibScalarInstance,) = mibBuilder.import_symbols("SNMPv2-SMI", "MibScalarInstance")

(
    snmpUnknownSecurityModels,
    snmpInvalidMsgs,
    snmpUnknownPDUHandlers,
) = mibBuilder.import_symbols(
    "SNMP-MPD-MIB",
    "snmpUnknownSecurityModels",
    "snmpInvalidMsgs",
    "snmpUnknownPDUHandlers",
)

__snmpUnknownSecurityModels = MibScalarInstance(
    snmpUnknownSecurityModels.name, (0,), snmpUnknownSecurityModels.syntax.clone(0)
)
__snmpInvalidMsgs = MibScalarInstance(
    snmpInvalidMsgs.name, (0,), snmpInvalidMsgs.syntax.clone(0)
)
__snmpUnknownPDUHandlers = MibScalarInstance(
    snmpUnknownPDUHandlers.name, (0,), snmpUnknownPDUHandlers.syntax.clone(0)
)

mibBuilder.export_symbols(
    "__SNMP-MPD-MIB",
    snmpUnknownSecurityModels=__snmpUnknownSecurityModels,
    snmpInvalidMsgs=__snmpInvalidMsgs,
    snmpUnknownPDUHandlers=__snmpUnknownPDUHandlers,
)



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
import os
import sys
import warnings

from pysnmp.smi.builder import MibBuilder

DEFAULT_SOURCES = ["file:///usr/share/snmp/mibs", "file:///usr/share/mibs"]

if sys.platform[:3] == "win":
    DEFAULT_DEST = os.path.join(os.path.expanduser("~"), "PySNMP Configuration", "mibs")
else:
    DEFAULT_DEST = os.path.join(os.path.expanduser("~"), ".pysnmp", "mibs")

DEFAULT_BORROWERS = []

try:
    from pysmi.reader.url import getReadersFromUrls
    from pysmi.searcher.pypackage import PyPackageSearcher
    from pysmi.searcher.stub import StubSearcher
    from pysmi.borrower.pyfile import PyFileBorrower
    from pysmi.writer.pyfile import PyFileWriter
    from pysmi.parser.smi import parserFactory
    from pysmi.parser.dialect import smiV1Relaxed
    from pysmi.codegen.pysnmp import PySnmpCodeGen, baseMibs
    from pysmi.compiler import MibCompiler

except ImportError:
    from pysnmp.smi import error

    def add_mib_compiler_decorator(errorMsg):
        """Add MIB compiler decorator."""

        def add_mib_compiler(mibBuilder, **kwargs):
            if not kwargs.get("ifAvailable"):
                raise error.SmiError("MIB compiler not available: %s" % errorMsg)

        return add_mib_compiler

    add_mib_compiler = add_mib_compiler_decorator(sys.exc_info()[1])

else:

    def add_mib_compiler(mibBuilder: MibBuilder, **kwargs):
        """Add MIB compiler to MIB builder."""
        if kwargs.get("ifNotAdded") and mibBuilder.get_mib_compiler():
            return

        compiler = MibCompiler(
            parserFactory(**smiV1Relaxed)(),
            PySnmpCodeGen(),
            PyFileWriter(kwargs.get("destination") or DEFAULT_DEST),
        )

        compiler.addSources(
            *getReadersFromUrls(*kwargs.get("sources") or DEFAULT_SOURCES)
        )

        compiler.addSearchers(StubSearcher(*baseMibs))
        compiler.addSearchers(
            *[PyPackageSearcher(x.full_path()) for x in mibBuilder.get_mib_sources()]
        )
        compiler.addBorrowers(
            *[
                PyFileBorrower(x, genTexts=mibBuilder.loadTexts)
                for x in getReadersFromUrls(
                    *kwargs.get("borrowers") or DEFAULT_BORROWERS,
                    **dict(lowcaseMatching=False),
                )
            ]
        )

        mibBuilder.set_mib_compiler(compiler, kwargs.get("destination") or DEFAULT_DEST)


# Compatibility API
deprecated_attributes = {
    "addMibCompiler": "add_mib_compiler",
    "addMibCompilerDecorator": "add_mib_compiler_decorator",
}


def __getattr__(attr: str):
    if new_attr := deprecated_attributes.get(attr):
        warnings.warn(
            f"{attr} is deprecated. Please use {new_attr} instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return globals()[new_attr]
    raise AttributeError(f"module '{__name__}' has no attribute '{attr}'")



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
from pyasn1.type import constraint, univ
from pysnmp.proto import errind, rfc1901, rfc1902, rfc1905
from pysnmp.proto.api import v1

# Shortcuts to SNMP types
Null = univ.Null
null = Null("")
ObjectIdentifier = univ.ObjectIdentifier

Integer = rfc1902.Integer
Integer32 = rfc1902.Integer32
OctetString = rfc1902.OctetString
IpAddress = rfc1902.IpAddress
Counter32 = rfc1902.Counter32
Gauge32 = rfc1902.Gauge32
Unsigned32 = rfc1902.Unsigned32
TimeTicks = rfc1902.TimeTicks
Opaque = rfc1902.Opaque
Counter64 = rfc1902.Counter64
Bits = rfc1902.Bits

NoSuchObject = rfc1905.NoSuchObject
NoSuchInstance = rfc1905.NoSuchInstance
EndOfMibView = rfc1905.EndOfMibView

VarBind = rfc1905.VarBind
VarBindList = rfc1905.VarBindList
GetRequestPDU = rfc1905.GetRequestPDU
GetNextRequestPDU = rfc1905.GetNextRequestPDU
ResponsePDU = GetResponsePDU = rfc1905.ResponsePDU
SetRequestPDU = rfc1905.SetRequestPDU
GetBulkRequestPDU = rfc1905.GetBulkRequestPDU
InformRequestPDU = rfc1905.InformRequestPDU
SNMPv2TrapPDU = TrapPDU = rfc1905.SNMPv2TrapPDU
ReportPDU = rfc1905.ReportPDU

Message = rfc1901.Message

getNextRequestID = v1.getNextRequestID  # noqa: N816

apiVarBind = v1.apiVarBind  # noqa: N816


class PDUAPI(v1.PDUAPI):
    """SNMPv2c request PDU API."""

    _error_status = rfc1905.errorStatus.clone(0)
    _error_index = univ.Integer(0).subtype(
        subtypeSpec=constraint.ValueRangeConstraint(0, rfc1905.max_bindings)
    )

    def get_response(self, reqPDU) -> rfc1905.ResponsePDU:
        """Build response PDU."""
        rspPDU = ResponsePDU()
        self.set_defaults(rspPDU)
        self.set_request_id(rspPDU, self.get_request_id(reqPDU))
        return rspPDU

    def get_varbind_table(self, reqPDU, rspPDU):
        """Get var-binds table from response PDU."""
        return [apiPDU.get_varbinds(rspPDU)]

    def get_next_varbinds(self, varBinds, origVarBinds=None):
        """Get next var-binds."""
        errorIndication = None
        idx = nonNulls = len(varBinds)
        rspVarBinds = []
        while idx:
            idx -= 1
            if varBinds[idx][1].tagSet in (
                rfc1905.NoSuchObject.tagSet,
                rfc1905.NoSuchInstance.tagSet,
                rfc1905.EndOfMibView.tagSet,
            ):
                nonNulls -= 1
            elif origVarBinds is not None:
                seed = ObjectIdentifier(origVarBinds[idx][0]).asTuple()
                found = varBinds[idx][0].asTuple()
                if seed >= found:
                    errorIndication = errind.oidNotIncreasing

            rspVarBinds.insert(0, (varBinds[idx][0], null))

        if not nonNulls:
            rspVarBinds = []

        return errorIndication, rspVarBinds

    def set_end_of_mib_error(self, pdu, errorIndex):
        """Set endOfMibView error."""
        varBindList = self.get_varbind_list(pdu)
        varBindList[errorIndex - 1].setComponentByPosition(
            1,
            rfc1905.endOfMibView,
            verifyConstraints=False,
            matchTags=False,
            matchConstraints=False,
        )

    def set_no_such_instance_error(self, pdu, errorIndex):
        """Set noSuchInstance error."""
        varBindList = self.get_varbind_list(pdu)
        varBindList[errorIndex - 1].setComponentByPosition(
            1,
            rfc1905.noSuchInstance,
            verifyConstraints=False,
            matchTags=False,
            matchConstraints=False,
        )


apiPDU = PDUAPI()  # noqa: N816


class BulkPDUAPI(PDUAPI):
    """SNMPv2c bulk request PDU API."""

    _non_repeaters = rfc1905.nonRepeaters.clone(0)
    _max_repetitions = rfc1905.maxRepetitions.clone(10)

    def set_defaults(self, pdu):
        """Set SNMPv2c bulk request defaults."""
        PDUAPI.set_defaults(self, pdu)
        pdu.setComponentByPosition(
            0,
            getNextRequestID(),
            verifyConstraints=False,
            matchTags=False,
            matchConstraints=False,
        )
        pdu.setComponentByPosition(
            1,
            self._non_repeaters,
            verifyConstraints=False,
            matchTags=False,
            matchConstraints=False,
        )
        pdu.setComponentByPosition(
            2,
            self._max_repetitions,
            verifyConstraints=False,
            matchTags=False,
            matchConstraints=False,
        )
        varBindList = pdu.setComponentByPosition(3).getComponentByPosition(3)
        varBindList.clear()

    @staticmethod
    def get_non_repeaters(pdu):
        """Get non-repeaters component of the PDU."""
        return pdu.getComponentByPosition(1)

    @staticmethod
    def set_non_repeaters(pdu, value):
        """Set non-repeaters component of the PDU."""
        pdu.setComponentByPosition(1, value)

    @staticmethod
    def get_max_repetitions(pdu):
        """Get max repetitions component of the PDU."""
        return pdu.getComponentByPosition(2)

    @staticmethod
    def set_max_repetitions(pdu, value):
        """Set max repetitions component of the PDU."""
        pdu.setComponentByPosition(2, value)

    def get_varbind_table(self, reqPDU, rspPDU):
        """Get var-binds table from response PDU."""
        nonRepeaters = self.get_non_repeaters(reqPDU)

        reqVarBinds = self.get_varbinds(reqPDU)

        N = min(int(nonRepeaters), len(reqVarBinds))

        rspVarBinds = self.get_varbinds(rspPDU)

        # shortcut for the most trivial case
        if N == 0 and len(reqVarBinds) == 1:
            return [[vb] for vb in rspVarBinds]

        R = max(len(reqVarBinds) - N, 0)

        varBindTable = []

        if R:
            for i in range(0, len(rspVarBinds) - N, R):
                varBindRow = rspVarBinds[:N] + rspVarBinds[N + i : N + R + i]
                # ignore stray OIDs / non-rectangular table
                if len(varBindRow) == N + R:
                    varBindTable.append(varBindRow)
        elif N:
            varBindTable.append(rspVarBinds[:N])

        return varBindTable


apiBulkPDU = BulkPDUAPI()  # noqa: N816


class TrapPDUAPI(v1.PDUAPI):
    """SNMPv2c trap PDU API."""

    sysUpTime = (1, 3, 6, 1, 2, 1, 1, 3, 0)  # noqa: N815
    snmpTrapAddress = (1, 3, 6, 1, 6, 3, 18, 1, 3, 0)  # noqa: N815
    snmpTrapCommunity = (1, 3, 6, 1, 6, 3, 18, 1, 4, 0)  # noqa: N815
    snmpTrapOID = (1, 3, 6, 1, 6, 3, 1, 1, 4, 1, 0)  # noqa: N815
    snmpTrapEnterprise = (1, 3, 6, 1, 6, 3, 1, 1, 4, 3, 0)  # noqa: N815
    _zeroTime = TimeTicks(0)  # noqa: N815
    _genTrap = ObjectIdentifier((1, 3, 6, 1, 6, 3, 1, 1, 5, 1))  # noqa: N815

    def set_defaults(self, pdu):
        """Set SNMPv2c trap defaults."""
        v1.PDUAPI.set_defaults(self, pdu)
        varBinds = [
            (self.sysUpTime, self._zeroTime),
            # generic trap
            (self.snmpTrapOID, self._genTrap),
        ]
        self.set_varbinds(pdu, varBinds)


apiTrapPDU = TrapPDUAPI()  # noqa: N816


class MessageAPI(v1.MessageAPI):
    """SNMPv2c message API."""

    _version = rfc1901.version.clone(1)

    def set_defaults(self, msg):
        """Set SNMP message defaults."""
        msg.setComponentByPosition(
            0,
            self._version,
            verifyConstraints=False,
            matchTags=False,
            matchConstraints=False,
        )
        msg.setComponentByPosition(
            1,
            self._community,
            verifyConstraints=False,
            matchTags=False,
            matchConstraints=False,
        )
        return msg

    def get_response(self, reqMsg) -> rfc1901.Message:
        """Build response message."""
        rspMsg = Message()
        self.set_defaults(rspMsg)
        self.set_version(rspMsg, self.get_version(reqMsg))
        self.set_community(rspMsg, self.get_community(reqMsg))
        self.set_pdu(rspMsg, apiPDU.get_response(self.get_pdu(reqMsg)))
        return rspMsg


apiMessage = MessageAPI()  # noqa: N816



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
import warnings
from time import time
from typing import Any

from pyasn1.codec.ber import decoder, encoder
from pysnmp import debug
from pysnmp.carrier.base import AbstractTransportDispatcher
from pysnmp.entity.engine import SnmpEngine
from pysnmp.hlapi.transport import AbstractTransportTarget
from pysnmp.proto import api
from pysnmp.proto import errind, error
from pysnmp.proto.api import verdec

__all__ = []


class AbstractSnmpDispatcher:
    """Creates SNMP message dispatcher object.

    `SnmpDispatcher` object manages send and receives SNMP PDU
    messages through underlying transport dispatcher and dispatches
    them to the callers.

    `SnmpDispatcher` is the only stateful object, all `hlapi.v1arch` SNMP
    operations require an instance of `SnmpDispatcher`. Users do not normally
    request services directly from `SnmpDispather`, but pass it around to
    other `hlapi.v1arch` interfaces.

    It is possible to run multiple instances of `SnmpDispatcher` in the
    application. In a multithreaded environment, each thread that
    works with SNMP must have its own `SnmpDispatcher` instance.
    """

    PROTO_DISPATCHER = None
    transport_dispatcher: AbstractTransportDispatcher
    cache: dict[str, Any]

    def __init__(self, transportDispatcher: AbstractTransportDispatcher = None):  # type: ignore
        if transportDispatcher:
            self.transport_dispatcher = transportDispatcher

        else:
            self.transport_dispatcher = self.PROTO_DISPATCHER()

        self._automaticDispatcher = transportDispatcher is not self.transport_dispatcher
        self._configuredTransports = set()

        self._pendingReqs = {}

        self.transport_dispatcher.register_recv_callback(self._recv_callback)
        self.transport_dispatcher.register_timer_callback(self._timer_callback)

        self.cache = {}

    def __repr__(self):
        return f"{self.__class__.__name__}(transportDispatcher={self.transport_dispatcher})"

    def close(self):
        self.transport_dispatcher.unregister_recv_callback()
        self.transport_dispatcher.unregister_timer_callback()
        if self._automaticDispatcher:
            self.transport_dispatcher.close_dispatcher()

        for requestId, stateInfo in self._pendingReqs.items():
            cbFun = stateInfo["cbFun"]
            cbCtx = stateInfo["cbCtx"]

            if cbFun:
                cbFun(self, "Request #%d terminated" % requestId, None, cbCtx)

        self._pendingReqs.clear()

    def send_pdu(
        self,
        authData,
        transportTarget: AbstractTransportTarget,
        reqPdu,
        cbFun=None,
        cbCtx=None,
    ):
        if (
            self._automaticDispatcher
            and transportTarget.TRANSPORT_DOMAIN not in self._configuredTransports
        ):
            self.transport_dispatcher.register_transport(
                transportTarget.TRANSPORT_DOMAIN,
                transportTarget.PROTO_TRANSPORT().open_client_mode(
                    transportTarget.iface
                ),
            )
            self._configuredTransports.add(transportTarget.TRANSPORT_DOMAIN)

        pMod = api.PROTOCOL_MODULES[authData.mpModel]

        reqMsg = pMod.Message()
        pMod.apiMessage.set_defaults(reqMsg)
        pMod.apiMessage.set_community(reqMsg, authData.communityName)
        pMod.apiMessage.set_pdu(reqMsg, reqPdu)

        outgoingMsg = encoder.encode(reqMsg)

        requestId = pMod.apiPDU.get_request_id(reqPdu)

        self._pendingReqs[requestId] = dict(
            outgoingMsg=outgoingMsg,
            transportTarget=transportTarget,
            cbFun=cbFun,
            cbCtx=cbCtx,
            timestamp=time() + transportTarget.timeout,
            retries=0,
        )

        self.transport_dispatcher.send_message(
            outgoingMsg,
            transportTarget.TRANSPORT_DOMAIN,
            transportTarget.transport_address,
        )

        if reqPdu.__class__ is getattr(
            pMod, "SNMPv2TrapPDU", None
        ) or reqPdu.__class__ is getattr(pMod, "TrapPDU", None):
            return requestId

        self.transport_dispatcher.job_started(id(self))

        return requestId

    def _recv_callback(
        self, snmpEngine: SnmpEngine, transportDomain, transportAddress, wholeMsg
    ):
        try:
            mpModel = verdec.decode_message_version(wholeMsg)

        except error.ProtocolError:
            return  # n.b the whole buffer gets dropped

        debug.logger & debug.FLAG_DSP and debug.logger(
            "receiveMessage: msgVersion %s, msg decoded" % mpModel
        )

        pMod = api.PROTOCOL_MODULES[mpModel]

        while wholeMsg:
            rspMsg, wholeMsg = decoder.decode(wholeMsg, asn1Spec=pMod.Message())
            rspPdu = pMod.apiMessage.get_pdu(rspMsg)

            requestId = pMod.apiPDU.get_request_id(rspPdu)

            try:
                stateInfo = self._pendingReqs.pop(requestId)

            except KeyError:
                continue

            self.transport_dispatcher.job_finished(id(self))

            cbFun = stateInfo["cbFun"]
            cbCtx = stateInfo["cbCtx"]

            if cbFun:
                cbFun(self, requestId, None, rspPdu, cbCtx)

        return wholeMsg

    def _timer_callback(self, timeNow):
        for requestId, stateInfo in tuple(self._pendingReqs.items()):
            if stateInfo["timestamp"] > timeNow:
                continue

            retries = stateInfo["retries"]
            transportTarget: AbstractTransportTarget = stateInfo["transportTarget"]

            if retries == transportTarget.retries:
                cbFun = stateInfo["cbFun"]
                cbCtx = stateInfo["cbCtx"]

                if cbFun:
                    del self._pendingReqs[requestId]
                    cbFun(
                        self,
                        requestId,
                        errind.requestTimedOut,
                        None,
                        cbCtx,
                    )
                    self.transport_dispatcher.job_finished(id(self))
                    continue

            stateInfo["retries"] += 1
            stateInfo["timestamp"] = timeNow + transportTarget.timeout

            outgoingMsg = stateInfo["outgoingMsg"]

            self.transport_dispatcher.send_message(
                outgoingMsg,
                transportTarget.TRANSPORT_DOMAIN,
                transportTarget.transport_address,
            )

    # compatibility with legacy code
    # Old to new attribute mapping
    deprecated_attributes = {
        "transportDispatcher": "transport_dispatcher",
        "sendPdu": "send_pdu",
    }

    def __getattr__(self, attr: str):
        if new_attr := self.deprecated_attributes.get(attr):
            warnings.warn(
                f"{attr} is deprecated. Please use {new_attr} instead.",
                DeprecationWarning,
                stacklevel=2,
            )
            return getattr(self, new_attr)
        raise AttributeError(
            f"'{self.__class__.__name__}' object has no attribute '{attr}'"
        )

    def __enter__(self):
        """Open the SNMP dispatcher."""
        return self

    def __exit__(self, *args, **kwargs):
        """Close the SNMP dispatcher."""
        self.close()



"""
SNMPv1 TRAP with defaults
+++++++++++++++++++++++++

Send SNMPv1 TRAP through unified SNMPv3 message processing framework
using the following options:

* SNMPv1
* with community name 'public'
* over IPv4/UDP
* send TRAP notification
* with Generic Trap #1 (warmStart) and Specific Trap 0
* with default Uptime
* with default Agent Address
* with Enterprise OID 1.3.6.1.4.1.20408.4.1.1.2
* include managed object information '1.3.6.1.2.1.1.1.0' = 'my system'

Functionally similar to:

| $ snmptrap -v1 -c public demo.pysnmp.com 1.3.6.1.4.1.20408.4.1.1.2 0.0.0.0 1 0 0 1.3.6.1.2.1.1.1.0 s "my system"

"""  #
import asyncio
from pysnmp.hlapi.v3arch.asyncio import *


async def run():
    snmpEngine = SnmpEngine()
    errorIndication, errorStatus, errorIndex, varBinds = await send_notification(
        snmpEngine,
        CommunityData("public", mpModel=0),
        await UdpTransportTarget.create(("demo.pysnmp.com", 162)),
        ContextData(),
        "trap",
        NotificationType(ObjectIdentity("1.3.6.1.6.3.1.1.5.2"))
        .load_mibs("SNMPv2-MIB")
        .add_varbinds(
            ("1.3.6.1.6.3.1.1.4.3.0", "1.3.6.1.4.1.20408.4.1.1.2"),
            ("1.3.6.1.2.1.1.1.0", OctetString("my system")),
        ),
    )

    if errorIndication:
        print(errorIndication)

    snmpEngine.close_dispatcher()


asyncio.run(run())



"""
Walk Agent MIB (SNMPv1)
+++++++++++++++++++++++

Perform SNMP GETNEXT operation with the following options:

* with SNMPv1, community 'public'
* over IPv4/UDP
* to an Agent at demo.pysnmp.com:161
* for OID in tuple form

This script performs similar to the following Net-SNMP command:

| $ snmpwalk -v1 -c public -ObentU demo.pysnmp.com 1.3.6

"""  #
from pysnmp.carrier.asyncio.dispatch import AsyncioDispatcher
from pysnmp.carrier.asyncio.dgram import udp
from pyasn1.codec.ber import encoder, decoder
from pysnmp.proto import api

# Protocol version to use
pMod = api.PROTOCOL_MODULES[api.SNMP_VERSION_1]
# pMod = api.PROTOCOL_MODULES[api.SNMP_VERSION_2C]

# SNMP table header
headVars = [pMod.ObjectIdentifier((1, 3, 6))]

# Build PDU
reqPDU = pMod.GetNextRequestPDU()
pMod.apiPDU.set_defaults(reqPDU)
pMod.apiPDU.set_varbinds(reqPDU, [(x, pMod.null) for x in headVars])

# Build message
reqMsg = pMod.Message()
pMod.apiMessage.set_defaults(reqMsg)
pMod.apiMessage.set_community(reqMsg, "public")
pMod.apiMessage.set_pdu(reqMsg, reqPDU)


# noinspection PyUnusedLocal
def cbRecvFun(
    transportDispatcher,
    transportDomain,
    transportAddress,
    wholeMsg,
    reqPDU=reqPDU,
    headVars=headVars,
):
    while wholeMsg:
        rspMsg, wholeMsg = decoder.decode(wholeMsg, asn1Spec=pMod.Message())
        rspPDU = pMod.apiMessage.get_pdu(rspMsg)

        # Match response to request
        if pMod.apiPDU.get_request_id(reqPDU) == pMod.apiPDU.get_request_id(rspPDU):
            # Check for SNMP errors reported
            errorStatus = pMod.apiPDU.get_error_status(rspPDU)
            if errorStatus and errorStatus != 2:
                raise Exception(errorStatus)

            # Format var-binds table
            varBindTable = pMod.apiPDU.get_varbind_table(reqPDU, rspPDU)

            # Report SNMP table
            for tableRow in varBindTable:
                for name, val in tableRow:
                    print(
                        "from: {}, {} = {}".format(
                            transportAddress, name.prettyPrint(), val.prettyPrint()
                        )
                    )

            # Stop on EOM
            for oid, val in varBindTable[-1]:
                if not isinstance(val, pMod.Null):
                    break

            else:
                transportDispatcher.job_finished(1)
                continue

            # Generate request for next row
            pMod.apiPDU.set_varbinds(
                reqPDU, [(x, pMod.null) for x, y in varBindTable[-1]]
            )

            pMod.apiPDU.set_request_id(reqPDU, pMod.get_next_request_id())

            transportDispatcher.send_message(
                encoder.encode(reqMsg), transportDomain, transportAddress
            )

    return wholeMsg


transportDispatcher = AsyncioDispatcher()

transportDispatcher.register_recv_callback(cbRecvFun)

transportDispatcher.register_transport(
    udp.DOMAIN_NAME, udp.UdpAsyncioTransport().open_client_mode()
)

transportDispatcher.send_message(
    encoder.encode(reqMsg), udp.DOMAIN_NAME, ("demo.pysnmp.com", 161)
)

transportDispatcher.job_started(1)

transportDispatcher.run_dispatcher(3)

transportDispatcher.close_dispatcher()



import asyncio
import socket
import pytest


from pysnmp.hlapi.v3arch.asyncio import *
from tests.manager_context import MANAGER_PORT, ManagerContextManager


@pytest.mark.asyncio
async def test_send_v3_trap_notification():
    async with ManagerContextManager() as (_, message_count):
        # snmptrap -v3 -l authPriv -u usr-md5-des -A authkey1 -X privkey1 -e 8000000001020304 localhost:MANAGER_PORT 0 1.3.6.1.6.3.1.1.5.1 1.3.6.1.2.1.1.1.0 s "my system"
        snmpEngine = SnmpEngine(OctetString(hexValue="8000000001020304"))
        errorIndication, errorStatus, errorIndex, varBinds = await send_notification(
            snmpEngine,
            UsmUserData("usr-md5-des", "authkey1", "privkey1"),
            await UdpTransportTarget.create(("localhost", MANAGER_PORT)),
            ContextData(),
            "trap",
            NotificationType(ObjectIdentity("IF-MIB", "linkDown")),
        )

        snmpEngine.close_dispatcher()
        await asyncio.sleep(1)
        assert message_count == [1]


@pytest.mark.asyncio
async def test_send_v3_trap_notification_none():
    async with ManagerContextManager() as (_, message_count):
        # snmptrap -v3 -l noAuthNoPriv -u usr-none-none -e 8000000001020305 localhost:MANAGER_PORT 0 1.3.6.1.6.3.1.1.5.1 1.3.6.1.2.1.1.1.0 s "my system"
        snmpEngine = SnmpEngine(OctetString(hexValue="8000000001020305"))
        errorIndication, errorStatus, errorIndex, varBinds = await send_notification(
            snmpEngine,
            UsmUserData("usr-none-none", None, None),
            await UdpTransportTarget.create(("localhost", MANAGER_PORT)),
            ContextData(),
            "trap",
            NotificationType(ObjectIdentity("IF-MIB", "linkDown")),
        )

        snmpEngine.close_dispatcher()
        await asyncio.sleep(1)
        assert message_count == [1]


@pytest.mark.asyncio
async def test_send_v3_trap_notification_invalid_user():
    async with ManagerContextManager() as (_, message_count):
        # snmptrap -v3 -l authPriv -u usr-md5-des -A authkey1 -X privkey1 -e 8000000001020304 localhost:MANAGER_PORT 0 1.3.6.1.6.3.1.1.5.1 1.3.6.1.2.1.1.1.0 s "my system"
        snmpEngine = SnmpEngine(OctetString(hexValue="8000000001020304"))
        errorIndication, errorStatus, errorIndex, varBinds = await send_notification(
            snmpEngine,
            UsmUserData("usr-md5-des1", "authkey1", "privkey1"),
            await UdpTransportTarget.create(("localhost", MANAGER_PORT)),
            ContextData(),
            "trap",
            NotificationType(ObjectIdentity("IF-MIB", "linkDown")),
        )

        snmpEngine.close_dispatcher()
        await asyncio.sleep(1)
        assert message_count == [0]


hex_dump = """
30 81 BC 02  01 03 30 11  02 04 61 03  A8 E3 02 03
00 FF E3 04  01 03 02 01  03 04 3A 30  38 04 08 80
00 00 00 01  02 03 04 02  01 01 02 04  05 35 84 3C
04 0B 75 73  72 2D 6D 64  35 2D 64 65  73 04 0C 36
04 72 CD 9B  57 02 49 45  C2 86 F6 04  08 00 00 00
0C CB C9 9A  08 04 68 2B  7D 70 58 59  12 E1 9F CC
A7 74 AD D7  1F 89 D4 BD  6B 8A D4 93  50 E5 31 82
98 45 C5 2A  23 CA 0A D2  3B A0 CF 59  C4 96 58 D3
CC 1A 8C 8F  87 82 FC 27  E4 AC 6B 54  8A 28 B9 D3
FA CD 92 E6  62 0C FB 65  42 62 9E CE  76 34 A9 02
3C CF 3A 05  7C 1F 3C B2  D3 0E B4 2F  8A 66 CB B4
10 66 A8 F4  A9 E3 D8 3C  44 BE 28 52  AC 70 27
"""
# copied from snmptrap -d -v3 -l authPriv -u usr-md5-des -A authkey1 -X privkey1 -e 8000000001020304 localhost:1622 0 1.3.6.1.6.3.1.1.5.1 1.3.6.1.2.1.1.1.0 s "my system"


def hex_dump_to_bytes(hex_dump):
    # Split the hex dump into lines
    lines = hex_dump.strip().split("\n")

    # Extract the hex bytes from each line
    hex_bytes = []
    for line in lines:
        # Skip the beginning of the line (offset) and drop the end of the line (non-hex characters)
        hex_part = line.replace(" ", "")
        hex_bytes.append(hex_part)

    # Join all hex bytes into a single string
    hex_string = "".join(hex_bytes)

    # Convert the hex string to a byte array
    return bytes.fromhex(hex_string)


@pytest.mark.asyncio
async def test_send_v3_trap_notification_raw():
    async with ManagerContextManager() as (_, message_count):
        raw_bytes = hex_dump_to_bytes(hex_dump)
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.sendto(raw_bytes, ("localhost", MANAGER_PORT))
        await asyncio.sleep(1)
        assert message_count == [1]


@pytest.mark.asyncio
async def test_send_v3_trap_notification_invalid_msg_flags():
    async with ManagerContextManager() as (_, message_count):
        raw_bytes = bytearray(hex_dump_to_bytes(hex_dump))
        assert raw_bytes[21] == 0x03  # check the original value
        raw_bytes[
            21
        ] = 0x02  # manually changed line 2, "04  01 03" to "04  01 02" to make the message invalid
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.sendto(raw_bytes, ("localhost", MANAGER_PORT))
        await asyncio.sleep(1)
        assert message_count == [0]



"""
Serve multiple network transports
+++++++++++++++++++++++++++++++++

Listen and respond to SNMP GET/SET/GETNEXT/GETBULK queries with
the following options:

* SNMPv2c
* with SNMP community "public"
* allow access to SNMPv2-MIB objects (1.3.6.1.2.1)
* over IPv4/UDP, listening at 127.0.0.1:161 and
  over IPv6/UDP, listening at [::1]:161

Either of the following Net-SNMP commands will walk this Agent:

| $ snmpwalk -v2c -c public 127.0.0.1 .1.3.6
| $ snmpwalk -v2c -c public udp6:[::1] .1.3.6

"""  #
from pysnmp.entity import engine, config
from pysnmp.entity.rfc3413 import cmdrsp, context
from pysnmp.carrier.asyncio.dgram import udp, udp6

# Create SNMP engine with autogenernated engineID and pre-bound
# to socket transport dispatcher
snmpEngine = engine.SnmpEngine()

# Transport setup

# UDP over IPv4 at 127.0.0.1:161
config.add_transport(
    snmpEngine, udp.DOMAIN_NAME, udp.UdpTransport().open_server_mode(("127.0.0.1", 161))
)
# UDP over IPv6 at [::1]:161
config.add_transport(
    snmpEngine, udp6.DOMAIN_NAME, udp6.Udp6Transport().open_server_mode(("::1", 161))
)

# SNMPv2c setup

# SecurityName <-> CommunityName mapping.
config.add_v1_system(snmpEngine, "my-area", "public")

# Allow full MIB access for this user / securityModels at VACM
config.add_vacm_user(
    snmpEngine, 2, "my-area", "noAuthNoPriv", (1, 3, 6, 1, 2, 1), (1, 3, 6, 1, 2, 1)
)

# Get default SNMP context this SNMP engine serves
snmpContext = context.SnmpContext(snmpEngine)

# Register SNMP Applications at the SNMP engine for particular SNMP context
cmdrsp.GetCommandResponder(snmpEngine, snmpContext)
cmdrsp.SetCommandResponder(snmpEngine, snmpContext)
cmdrsp.NextCommandResponder(snmpEngine, snmpContext)
cmdrsp.BulkCommandResponder(snmpEngine, snmpContext)

# Register an imaginary never-ending job to keep I/O dispatcher running forever
snmpEngine.transport_dispatcher.job_started(1)

# Run I/O dispatcher which would receive queries and send responses
try:
    snmpEngine.open_dispatcher()
except:
    snmpEngine.close_dispatcher()
    raise



import pytest
from pysnmp.hlapi.v3arch.asyncio import *
from pysnmp.smi.builder import MibBuilder
from pysnmp.smi.view import MibViewController
from tests.agent_context import AGENT_PORT, AgentContextManager

total_count = 212  # 267


@pytest.mark.asyncio
async def test_v1_walk():
    async with AgentContextManager():
        with SnmpEngine() as snmpEngine:
            objects = walk_cmd(
                snmpEngine,
                CommunityData("public", mpModel=0),
                await UdpTransportTarget.create(("localhost", AGENT_PORT)),
                ContextData(),
                ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
            )

            objects_list = [item async for item in objects]

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[0]

            assert errorIndication is None
            assert errorStatus == 0
            assert len(varBinds) == 1
            assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysObjectID.0"

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[1]

            assert errorIndication is None
            assert errorStatus == 0
            assert len(varBinds) == 1
            assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysUpTime.0"

            assert len(objects_list) == total_count


@pytest.mark.asyncio
async def test_v1_walk_mib():
    async with AgentContextManager():
        mib_builder = MibBuilder()
        mib_view_controller = MibViewController(mib_builder)
        mib_builder.load_modules(
            "SNMP-COMMUNITY-MIB",
            "PYSNMP-MIB",
            "PYSNMP-USM-MIB",
            "SNMP-VIEW-BASED-ACM-MIB",
        )

        with SnmpEngine() as snmpEngine:
            snmpEngine.cache["mibViewController"] = mib_view_controller
            objects = walk_cmd(
                snmpEngine,
                CommunityData("public", mpModel=0),
                await UdpTransportTarget.create(("localhost", AGENT_PORT)),
                ContextData(),
                ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
            )

            objects_list = [item async for item in objects]

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[0]

            assert errorIndication is None
            assert errorStatus == 0
            assert len(varBinds) == 1
            assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysObjectID.0"

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[1]

            assert errorIndication is None
            assert errorStatus == 0
            assert len(varBinds) == 1
            assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysUpTime.0"

            assert len(objects_list) == total_count

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[-1]
            assert (
                varBinds[0][0].prettyPrint()
                == 'SNMP-COMMUNITY-MIB::snmpCommunityStatus."public"'
            )

            for errorIndication, errorStatus, errorIndex, varBinds in objects_list:
                content = varBinds[0][0].prettyPrint()
                if (
                    not content.startswith("PYSNMP-USM-MIB::")
                    and not content.startswith("SNMP-USER-BASED-SM-MIB::")
                    and not content.startswith("SNMP-VIEW-BASED-ACM-MIB::")
                ):
                    assert content.count(".") == 1  # fully resolved.


@pytest.mark.asyncio
async def test_v1_walk_subtree():
    async with AgentContextManager():
        with SnmpEngine() as snmpEngine:
            objects = walk_cmd(
                snmpEngine,
                CommunityData("public", mpModel=0),
                await UdpTransportTarget.create(("localhost", AGENT_PORT)),
                ContextData(),
                ObjectType(ObjectIdentity("SNMPv2-MIB", "system")),
                lexicographicMode=False,
            )

            objects_list = [item async for item in objects]

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[0]

            assert errorIndication is None
            assert errorStatus == 0
            assert len(varBinds) == 1
            assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysDescr.0"

            errorIndication, errorStatus, errorIndex, varBinds = objects_list[1]

            assert errorIndication is None
            assert errorStatus == 0
            assert len(varBinds) == 1
            assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysObjectID.0"

            assert len(objects_list) == 8



"""
Fetch scalar and table variables
++++++++++++++++++++++++++++++++

Send a series of SNMP GETBULK requests with the following options:

* with SNMPv3 with user 'usr-md5-des', MD5 auth and DES privacy protocols
* over IPv4/UDP
* to an Agent at 127.0.0.1:161
* with values non-repeaters = 1, max-repetitions = 25
* for two OIDs in tuple form (first OID is non-repeating)
* stop on end-of-mib condition for both OIDs

This script performs similar to the following Net-SNMP command:

| $ snmpbulkwalk -v3 -l authPriv -u usr-md5-des -A authkey1 -X privkey1 -C n1 -C r25 -ObentU 127.0.0.1 1.3.6.1.2.1.1 1.3.6.1.4.1.1

"""  #

from pysnmp.entity import engine, config
from pysnmp.entity.rfc3413 import cmdgen
from pysnmp.carrier.asyncio.dgram import udp

# Create SNMP engine instance
snmpEngine = engine.SnmpEngine()

#
# SNMPv3/USM setup
#

# user: usr-md5-des, auth: MD5, priv DES
config.add_v3_user(
    snmpEngine,
    "usr-md5-des",
    config.USM_AUTH_HMAC96_MD5,
    "authkey1",
    config.USM_PRIV_CBC56_DES,
    "privkey1",
)
config.add_target_parameters(snmpEngine, "my-creds", "usr-md5-des", "authPriv")

#
# Setup transport endpoint and bind it with security settings yielding
# a target name
#

# UDP/IPv4
config.add_transport(
    snmpEngine, udp.DOMAIN_NAME, udp.UdpAsyncioTransport().open_client_mode()
)
config.add_target_address(
    snmpEngine, "my-router", udp.DOMAIN_NAME, ("127.0.0.1", 161), "my-creds"
)


# Error/response receiver
# noinspection PyUnusedLocal,PyUnusedLocal,PyUnusedLocal
def cbFun(
    snmpEngine,
    sendRequesthandle,
    errorIndication,
    errorStatus,
    errorIndex,
    varBindTable,
    cbCtx,
):
    if errorIndication:
        print(errorIndication)
        return  # stop on error
    if errorStatus:
        print(
            f"{errorStatus.prettyPrint()} at {varBindTable[-1][int(errorIndex) - 1][0] if errorIndex else '?'}"
        )
        return  # stop on error
    for varBindRow in varBindTable:
        for oid, val in varBindRow:
            print(f"{oid.prettyPrint()} = {val.prettyPrint()}")
    return True  # signal dispatcher to continue walking


# Prepare initial request to be sent
cmdgen.BulkCommandGenerator().send_varbinds(
    snmpEngine,
    "my-router",
    None,
    "",  # contextEngineId, contextName
    0,
    25,  # non-repeaters, max-repetitions
    (((1, 3, 6, 1, 2, 1, 1), None), ((1, 3, 6, 1, 4, 1, 1), None)),
    cbFun,
)

# Run I/O dispatcher which would send pending queries and process responses
snmpEngine.open_dispatcher(3)

snmpEngine.close_dispatcher()



# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
"""
This module initializes the SNMP protocol API for different versions.

It imports the necessary submodules for SNMP version 1 and version 2c, and
defines constants for these versions. It also provides a mapping of protocol
modules to their respective versions and exposes a function to decode the
message version.

Attributes:
 * SNMP_VERSION_1 (int): Constant for SNMP version 1.
 * SNMP_VERSION_2C (int): Constant for SNMP version 2c.
 * PROTOCOL_MODULES (dict): Mapping of SNMP version constants to their
        respective protocol modules.
 * decodeMessageVersion (function): Function to decode the message version.
"""
from pysnmp.proto.api import v1, v2c, verdec

# Protocol versions
SNMP_VERSION_1 = 0
SNMP_VERSION_2C = 1
PROTOCOL_MODULES = {SNMP_VERSION_1: v1, SNMP_VERSION_2C: v2c}

decodeMessageVersion = verdec.decode_message_version  # noqa: N816



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
"""
This module initializes the SNMP USM Security Model for the pysnmp library.

It imports the `SnmpUSMSecurityModel` from the `service` module within the
`pysnmp.proto.secmod.rfc3414` package.

Classes:
    SnmpUSMSecurityModel: A class representing the SNMP User-based Security Model (USM).

"""
from pysnmp.proto.secmod.rfc3414 import service

SnmpUSMSecurityModel = service.SnmpUSMSecurityModel



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
from pysnmp.carrier.asyncio.dispatch import AsyncioDispatcher
from pysnmp.hlapi.v1arch.dispatch import AbstractSnmpDispatcher

__all__ = ["SnmpDispatcher"]


class SnmpDispatcher(AbstractSnmpDispatcher):
    """Creates SNMP message dispatcher object.

    `SnmpDispatcher` object manages send and receives SNMP PDU
    messages through underlying transport dispatcher and dispatches
    them to the callers.

    `SnmpDispatcher` is the only stateful object, all `hlapi.v1arch` SNMP
    operations require an instance of `SnmpDispatcher`. Users do not normally
    request services directly from `SnmpDispather`, but pass it around to
    other `hlapi.v1arch` interfaces.

    It is possible to run multiple instances of `SnmpDispatcher` in the
    application. In a multithreaded environment, each thread that
    works with SNMP must have its own `SnmpDispatcher` instance.

    The `SnmpDispatcher` class supports context manager protocol,
    so it can be used with the `with` statement for automatic
    resource management:

    .. code-block:: python

       with SnmpDispatcher() as dispatcher:
           # use dispatcher
           pass  # dispatcher.close() is called automatically
    """

    PROTO_DISPATCHER = AsyncioDispatcher



"""
SNMP MIB browser
++++++++++++++++

This script explains how Python application (typically SNMP Manager)
could load SNMP MIB modules into memory and introspect Managed Objects
defined in MIB.

"""  #
from pysnmp.smi import builder, view, compiler, error

# Create MIB loader/builder
mibBuilder = builder.MibBuilder()

# Optionally attach PySMI MIB compiler (if installed)
# print('Attaching MIB compiler...'),
# compiler.addMibCompiler(mibBuilder, sources=['/usr/share/snmp/mibs'])
# print('done')

# Optionally set an alternative path to compiled MIBs
print("Setting MIB sources...")
mibBuilder.add_mib_sources(builder.DirMibSource("/opt/pysnmp_mibs"))
print(mibBuilder.get_mib_sources())
print("done")

print("Loading MIB modules..."),
mibBuilder.load_modules(
    "SNMPv2-MIB", "SNMP-FRAMEWORK-MIB", "SNMP-COMMUNITY-MIB", "IP-MIB"
)
print("done")

print("Indexing MIB objects..."),
mibView = view.MibViewController(mibBuilder)
print("done")

print("MIB symbol name lookup by OID: "),
oid, label, suffix = mibView.get_node_name((1, 3, 6, 1, 2, 1, 1, 1))
print(oid, label, suffix)

print("MIB symbol name lookup by label: "),
oid, label, suffix = mibView.get_node_name((1, 3, 6, 1, 2, "mib-2", 1, "sysDescr"))
print(oid, label, suffix)

print("MIB symbol name lookup by symbol description: "),
oid, label, suffix = mibView.get_node_name(("sysDescr",))
oid, label, suffix = mibView.get_node_name(("snmpEngineID",), "SNMP-FRAMEWORK-MIB")
print(oid, label, suffix)

print("MIB object value pretty print: "),
(mibNode,) = mibBuilder.import_symbols("SNMP-FRAMEWORK-MIB", "snmpEngineID")
print(mibNode.syntax.prettyPrint())

print("MIB symbol location lookup by name: "),
modName, symName, suffix = mibView.get_node_location(("snmpCommunityEntry",))
print(symName, modName)

print("MIB node lookup by location: "),
(rowNode,) = mibBuilder.import_symbols(modName, symName)
print(rowNode)

print("Conceptual table index value to oid conversion: "),
oid = rowNode.getInstIdFromIndices("router")
print(oid)
print("Conceptual table index oid to value conversion: "),
print(rowNode.getIndicesFromInstId(oid))

print("MIB tree traversal")
oid, label, suffix = mibView.get_first_node_name()
while 1:
    try:
        modName, nodeDesc, suffix = mibView.get_node_location(oid)
        print(f"{modName}::{nodeDesc} == {oid}")
        oid, label, suffix = mibView.get_next_node_name(oid)
    except error.NoSuchObjectError:
        break

print("Modules traversal")
modName = mibView.get_first_module_name()
while 1:
    if modName:
        print(modName)
    try:
        modName = mibView.get_next_module_name(modName)
    except error.SmiError:
        break



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# Copyright (c) 2022-2024, LeXtudio Inc. <support@lextudio.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
"""
This module provides constants and backward-compatible protocol IDs for SNMP authentication and privacy protocols, as well as key material types used in the User-based Security Model (USM).

Constants:
    USM_AUTH_NONE: No Authentication Protocol.
    USM_AUTH_HMAC96_MD5: The HMAC-MD5-96 Digest Authentication Protocol.
    USM_AUTH_HMAC96_SHA: The HMAC-SHA-96 Digest Authentication Protocol AKA SHA-1.
    USM_AUTH_HMAC128_SHA224: The HMAC-SHA-2 Digest Authentication Protocols.
    USM_AUTH_HMAC192_SHA256: The HMAC-SHA-2 Digest Authentication Protocols.
    USM_AUTH_HMAC256_SHA384: The HMAC-SHA-2 Digest Authentication Protocols.
    USM_AUTH_HMAC384_SHA512: The HMAC-SHA-2 Digest Authentication Protocols.
    USM_PRIV_NONE: No Privacy Protocol.
    USM_PRIV_CBC56_DES: The CBC-DES Symmetric Encryption Protocol.
    USM_PRIV_CBC168_3DES: The 3DES-EDE Symmetric Encryption Protocol.
    USM_PRIV_CFB128_AES: The CFB128-AES-128 Symmetric Encryption Protocol.
    USM_PRIV_CFB192_AES: The CFB128-AES-192 Symmetric Encryption Protocol with Reeder key localization.
    USM_PRIV_CFB256_AES: The CFB128-AES-256 Symmetric Encryption Protocol with Reeder key localization.
    USM_PRIV_CFB192_AES_BLUMENTHAL: The CFB128-AES-192 Symmetric Encryption Protocol.
    USM_PRIV_CFB256_AES_BLUMENTHAL: The CFB128-AES-256 Symmetric Encryption Protocol.
    USM_KEY_TYPE_PASSPHRASE: USM key material type - plain-text pass phrase.
    USM_KEY_TYPE_MASTER: USM key material type - hashed pass-phrase AKA master key.
    USM_KEY_TYPE_LOCALIZED: USM key material type - hashed pass-phrase hashed with Context SNMP Engine ID.

Backward-compatible protocol IDs:
    usmNoAuthProtocol: Alias for USM_AUTH_NONE.
    usmHMACMD5AuthProtocol: Alias for USM_AUTH_HMAC96_MD5.
    usmHMACSHAAuthProtocol: Alias for USM_AUTH_HMAC96_SHA.
    usmHMAC128SHA224AuthProtocol: Alias for USM_AUTH_HMAC128_SHA224.
    usmHMAC192SHA256AuthProtocol: Alias for USM_AUTH_HMAC192_SHA256.
    usmHMAC256SHA384AuthProtocol: Alias for USM_AUTH_HMAC256_SHA384.
    usmHMAC384SHA512AuthProtocol: Alias for USM_AUTH_HMAC384_SHA512.
    usmNoPrivProtocol: Alias for USM_PRIV_NONE.
    usmDESPrivProtocol: Alias for USM_PRIV_CBC56_DES.
    usm3DESEDEPrivProtocol: Alias for USM_PRIV_CBC168_3DES.
    usmAesCfb128Protocol: Alias for USM_PRIV_CFB128_AES.
    usmAesCfb192Protocol: Alias for USM_PRIV_CFB192_AES.
    usmAesCfb256Protocol: Alias for USM_PRIV_CFB256_AES.
    usmAesBlumenthalCfb192Protocol: Alias for USM_PRIV_CFB192_AES_BLUMENTHAL.
    usmAesBlumenthalCfb256Protocol: Alias for USM_PRIV_CFB256_AES_BLUMENTHAL.
    usmKeyTypePassphrase: Alias for USM_KEY_TYPE_PASSPHRASE.
    usmKeyTypeMaster: Alias for USM_KEY_TYPE_MASTER.
    usmKeyTypeLocalized: Alias for USM_KEY_TYPE_LOCALIZED.

Deprecated Attributes:
    A dictionary mapping old attribute names to new attribute names.

Functions:
    __getattr__(attr: str):
        Handles access to deprecated attributes, issuing a warning and returning
        the new attribute if available, or raising an AttributeError if not.
"""
import warnings

from pysnmp.entity.engine import *
from pysnmp.hlapi.v3arch.asyncio import auth
from pysnmp.hlapi.v3arch.asyncio.auth import CommunityData, UsmUserData
from pysnmp.hlapi.v3arch.asyncio.cmdgen import *
from pysnmp.hlapi.v3arch.asyncio.context import *
from pysnmp.hlapi.v3arch.asyncio.ntforg import *
from pysnmp.hlapi.v3arch.asyncio.transport import *
from pysnmp.proto.rfc1902 import *
from pysnmp.proto.rfc1905 import EndOfMibView, NoSuchInstance, NoSuchObject
from pysnmp.smi.rfc1902 import *

USM_AUTH_NONE = auth.USM_AUTH_NONE
"""No Authentication Protocol"""

USM_AUTH_HMAC96_MD5 = auth.USM_AUTH_HMAC96_MD5
"""The HMAC-MD5-96 Digest Authentication Protocol (:RFC:`3414#section-6`)"""

USM_AUTH_HMAC96_SHA = auth.USM_AUTH_HMAC96_SHA
"""The HMAC-SHA-96 Digest Authentication Protocol AKA SHA-1 \
(:RFC:`3414#section-7`)"""

USM_AUTH_HMAC128_SHA224 = auth.USM_AUTH_HMAC128_SHA224
"""The HMAC-SHA-2 Digest Authentication Protocols (:RFC:`7860`)"""

USM_AUTH_HMAC192_SHA256 = auth.USM_AUTH_HMAC192_SHA256
"""The HMAC-SHA-2 Digest Authentication Protocols (:RFC:`7860`)"""

USM_AUTH_HMAC256_SHA384 = auth.USM_AUTH_HMAC256_SHA384
"""The HMAC-SHA-2 Digest Authentication Protocols (:RFC:`7860`)"""

USM_AUTH_HMAC384_SHA512 = auth.USM_AUTH_HMAC384_SHA512
"""The HMAC-SHA-2 Digest Authentication Protocols (:RFC:`7860`)"""

USM_PRIV_NONE = auth.USM_PRIV_NONE
"""No Privacy Protocol"""

USM_PRIV_CBC56_DES = auth.USM_PRIV_CBC56_DES
"""The CBC-DES Symmetric Encryption Protocol (:RFC:`3414#section-8`)"""

USM_PRIV_CBC168_3DES = auth.USM_PRIV_CBC168_3DES
"""The 3DES-EDE Symmetric Encryption Protocol (`draft-reeder-snmpv3-usm-3desede-00 \
<https:://tools.ietf.org/html/draft-reeder-snmpv3-usm-3desede-00#section-5>`_)"""

USM_PRIV_CFB128_AES = auth.USM_PRIV_CFB128_AES
"""The CFB128-AES-128 Symmetric Encryption Protocol (:RFC:`3826#section-3`)"""

USM_PRIV_CFB192_AES = auth.USM_PRIV_CFB192_AES
"""The CFB128-AES-192 Symmetric Encryption Protocol (`draft-blumenthal-aes-usm-04 \
<https:://tools.ietf.org/html/draft-blumenthal-aes-usm-04#section-3>`_) with \
Reeder key localization. Also known as AES-192-Cisco"""

USM_PRIV_CFB256_AES = auth.USM_PRIV_CFB256_AES
"""The CFB128-AES-256 Symmetric Encryption Protocol (`draft-blumenthal-aes-usm-04 \
<https:://tools.ietf.org/html/draft-blumenthal-aes-usm-04#section-3>`_) with \
Reeder key localization. Also also known as AES-256-Cisco"""

USM_PRIV_CFB192_AES_BLUMENTHAL = auth.USM_PRIV_CFB192_AES_BLUMENTHAL
"""The CFB128-AES-192 Symmetric Encryption Protocol (`draft-blumenthal-aes-usm-04 \
<https:://tools.ietf.org/html/draft-blumenthal-aes-usm-04#section-3>`_)"""

USM_PRIV_CFB256_AES_BLUMENTHAL = auth.USM_PRIV_CFB256_AES_BLUMENTHAL
"""The CFB128-AES-256 Symmetric Encryption Protocol (`draft-blumenthal-aes-usm-04 \
<https:://tools.ietf.org/html/draft-blumenthal-aes-usm-04#section-3>`_)"""

USM_KEY_TYPE_PASSPHRASE = auth.USM_KEY_TYPE_PASSPHRASE
"""USM key material type - plain-text pass phrase (:RFC:`3414#section-2.6`)"""

USM_KEY_TYPE_MASTER = auth.USM_KEY_TYPE_MASTER
"""USM key material type - hashed pass-phrase AKA master key \
(:RFC:`3414#section-2.6`)"""

USM_KEY_TYPE_LOCALIZED = auth.USM_KEY_TYPE_LOCALIZED
"""USM key material type - hashed pass-phrase hashed with Context SNMP Engine \
ID (:RFC:`3414#section-2.6`)"""

# Backward-compatible protocol IDs

usmNoAuthProtocol = USM_AUTH_NONE  # noqa: N816
usmHMACMD5AuthProtocol = USM_AUTH_HMAC96_MD5  # noqa: N816
usmHMACSHAAuthProtocol = USM_AUTH_HMAC96_SHA  # noqa: N816
usmHMAC128SHA224AuthProtocol = USM_AUTH_HMAC128_SHA224  # noqa: N816
usmHMAC192SHA256AuthProtocol = USM_AUTH_HMAC192_SHA256  # noqa: N816
usmHMAC256SHA384AuthProtocol = USM_AUTH_HMAC256_SHA384  # noqa: N816
usmHMAC384SHA512AuthProtocol = USM_AUTH_HMAC384_SHA512  # noqa: N816
usmNoPrivProtocol = USM_PRIV_NONE  # noqa: N816
usmDESPrivProtocol = USM_PRIV_CBC56_DES  # noqa: N816
usm3DESEDEPrivProtocol = USM_PRIV_CBC168_3DES  # noqa: N816
usmAesCfb128Protocol = USM_PRIV_CFB128_AES  # noqa: N816
usmAesCfb192Protocol = USM_PRIV_CFB192_AES  # noqa: N816
usmAesCfb256Protocol = USM_PRIV_CFB256_AES  # noqa: N816
usmAesBlumenthalCfb192Protocol = USM_PRIV_CFB192_AES_BLUMENTHAL  # noqa: N816
usmAesBlumenthalCfb256Protocol = USM_PRIV_CFB256_AES_BLUMENTHAL  # noqa: N816

usmKeyTypePassphrase = USM_KEY_TYPE_PASSPHRASE  # noqa: N816
usmKeyTypeMaster = USM_KEY_TYPE_MASTER  # noqa: N816
usmKeyTypeLocalized = USM_KEY_TYPE_LOCALIZED  # noqa: N816

# Old to new attribute mapping
deprecated_attributes = {
    "usmNoAuthProtocol": "USM_AUTH_NONE",
    "usmHMACMD5AuthProtocol": "USM_AUTH_HMAC96_MD5",
    "usmHMACSHAAuthProtocol": "USM_AUTH_HMAC96_SHA",
    "usmHMAC128SHA224AuthProtocol": "USM_AUTH_HMAC128_SHA224",
    "usmHMAC192SHA256AuthProtocol": "USM_AUTH_HMAC192_SHA256",
    "usmHMAC256SHA384AuthProtocol": "USM_AUTH_HMAC256_SHA384",
    "usmHMAC384SHA512AuthProtocol": "USM_AUTH_HMAC384_SHA512",
    "usmNoPrivProtocol": "USM_PRIV_NONE",
    "usmDESPrivProtocol": "USM_PRIV_CBC56_DES",
    "usm3DESEDEPrivProtocol": "USM_PRIV_CBC168_3DES",
    "usmAesCfb128Protocol": "USM_PRIV_CFB128_AES",
    "usmAesCfb192Protocol": "USM_PRIV_CFB192_AES",
    "usmAesCfb256Protocol": "USM_PRIV_CFB256_AES",
    "usmAesBlumenthalCfb192Protocol": "USM_PRIV_CFB192_AES_BLUMENTHAL",
    "usmAesBlumenthalCfb256Protocol": "USM_PRIV_CFB256_AES_BLUMENTHAL",
    "usmKeyTypePassphrase": "USM_KEY_TYPE_PASSPHRASE",
    "usmKeyTypeMaster": "USM_KEY_TYPE_MASTER",
    "usmKeyTypeLocalized": "USM_KEY_TYPE_LOCALIZED",
}


def __getattr__(attr: str):
    if new_attr := deprecated_attributes.get(attr):
        warnings.warn(
            f"{attr} is deprecated. Please use {new_attr} instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return globals()[new_attr]
    raise AttributeError(f"module '{__name__}' has no attribute '{attr}'")



"""
Send packets from specific local interface
++++++++++++++++++++++++++++++++++++++++++

Send a series of SNMP GETNEXT requests with the following options:

* with SNMPv2c, community 'public'
* over IPv4/UDP
* to an Agent at 127.0.0.1:161
* sending packets from primary local interface 0.0.0.0, local port 61024
* for two OIDs in tuple form
* stop on end-of-mib condition for both OIDs

This script performs similar to the following Net-SNMP command:

| $ snmpwalk -v2c -c public -ObentU 127.0.0.1 1.3.6.1.2.1.1 1.3.6.1.4.1.1

"""  #

from pysnmp.entity import engine, config
from pysnmp.carrier.asyncio.dgram import udp
from pysnmp.entity.rfc3413 import cmdgen

# Create SNMP engine instance
snmpEngine = engine.SnmpEngine()

#
# SNMPv1/2c setup (if you use SNMPv1 or v2c)
#

# SecurityName <-> CommunityName mapping
config.add_v1_system(snmpEngine, "my-area", "public")

# Specify security settings per SecurityName (SNMPv1 - 0, SNMPv2c - 1)
config.add_target_parameters(snmpEngine, "my-creds", "my-area", "noAuthNoPriv", 0)

#
# Setup transport endpoint and bind it with security settings yielding
# a target name
#

# UDP/IPv4
config.add_transport(
    snmpEngine,
    udp.DOMAIN_NAME,
    udp.UdpAsyncioTransport().open_client_mode(("0.0.0.0", 61024)),
)
config.add_target_address(
    snmpEngine, "my-router", udp.DOMAIN_NAME, ("127.0.0.1", 161), "my-creds"
)


# Error/response receiver
# noinspection PyUnusedLocal,PyUnusedLocal,PyUnusedLocal
def cbFun(
    snmpEngine,
    sendRequestHandle,
    errorIndication,
    errorStatus,
    errorIndex,
    varBindTable,
    cbCtx,
):
    if errorIndication:
        print(errorIndication)
        return
    if errorStatus:
        print(
            f"{errorStatus.prettyPrint()} at {varBindTable[-1][int(errorIndex) - 1][0] or '?'}"
        )
        return  # stop on error
    for varBindRow in varBindTable:
        for oid, val in varBindRow:
            print(f"{oid.prettyPrint()} = {val.prettyPrint()}")
    return 1  # signal dispatcher to continue


# Prepare initial request to be sent
cmdgen.NextCommandGenerator().send_varbinds(
    snmpEngine,
    "my-router",
    None,
    "",  # contextEngineId, contextName
    [((1, 3, 6, 1, 2, 1, 1), None), ((1, 3, 6, 1, 2, 1, 11), None)],
    cbFun,
)

# Run I/O dispatcher which would send pending queries and process responses
snmpEngine.open_dispatcher(3)

snmpEngine.close_dispatcher()



"""
Concurrent queries
++++++++++++++++++

Send multiple SNMP GET requests at once using the following options:

* with SNMPv2c, community 'public'
* over IPv4/UDP
* to multiple Agents at demo.pysnmp.com
* for instance of SNMPv2-MIB::sysDescr.0 MIB object
* based on asyncio I/O framework

Functionally similar to:

| $ snmpget -v2c -c public demo.pysnmp.com:161 SNMPv2-MIB::sysDescr.0
| $ snmpget -v2c -c public demo.pysnmp.com:161 SNMPv2-MIB::sysDescr.0
| $ snmpget -v2c -c public demo.pysnmp.com:161 SNMPv2-MIB::sysDescr.0

"""  #
import asyncio
from pysnmp.hlapi.v3arch.asyncio import *


async def getone(snmpEngine, hostname):
    errorIndication, errorStatus, errorIndex, varBinds = await get_cmd(
        snmpEngine,
        CommunityData("public"),
        await UdpTransportTarget.create(hostname),
        ContextData(),
        ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
    )

    if errorIndication:
        print(errorIndication)
    elif errorStatus:
        print(
            "{} at {}".format(
                errorStatus.prettyPrint(),
                errorIndex and varBinds[int(errorIndex) - 1][0] or "?",
            )
        )
    else:
        for varBind in varBinds:
            print(" = ".join([x.prettyPrint() for x in varBind]))


async def main():
    snmpEngine = SnmpEngine()
    await asyncio.gather(
        getone(snmpEngine, ("demo.pysnmp.com", 161)),
        getone(snmpEngine, ("demo.pysnmp.com", 161)),
        getone(snmpEngine, ("demo.pysnmp.com", 161)),
    )


asyncio.run(main())



#!/usr/bin/env python3
"""
Script to build Python packages for pysmi.
Equivalent to Build-Packages.ps1
"""

import os
import shutil
import subprocess


def main():
    """Main function to build packages."""
    # Check if the dist directory exists and remove it
    if os.path.isdir("dist"):
        print("Removing existing dist directory...")
        shutil.rmtree("dist")
    else:
        print("dist directory not found. Skipping removal.")

    # Build the packages
    print("Building packages...")
    subprocess.run(["uv", "build"], check=True)

    print("Build process completed.")


if __name__ == "__main__":
    main()



"""
SNMPv3 TRAP, auth: MD5, privacy: DES
++++++++++++++++++++++++++++++++++++

Send SNMP TRAP notification using the following options:

* SNMPv3
* with user 'usr-md5-des', auth: MD5, priv DES
* over IPv4/UDP
* send TRAP notification
* to a Manager at 127.0.0.1:162
* with TRAP ID 'warmStart' specified as an OID
* include managed object information 1.3.6.1.2.1.1.5.0 = 'system name'

Functionally similar to:

| $ snmptrap -v3 -l authPriv -u usr-md5-des -A authkey1 -X privkey1 -e 8000000001020304 demo.pysnmp.com 0 1.3.6.1.6.3.1.1.5.1 1.3.6.1.2.1.1.1.0 s "my system"

"""  #
from pysnmp.entity import engine, config
from pysnmp.carrier.asyncio.dgram import udp
from pysnmp.entity.rfc3413 import ntforg
from pysnmp.proto.api import v2c

# Create SNMP engine instance with specific (and locally unique)
# SnmpEngineId -- it must also be known to the receiving party
# and configured at its VACM users table.
snmpEngine = engine.SnmpEngine(
    snmpEngineID=v2c.OctetString(hexValue="8000000001020304")
)

# Add USM user
config.add_v3_user(
    snmpEngine,
    "usr-md5-des",
    config.USM_AUTH_HMAC96_MD5,
    "authkey1",
    config.USM_PRIV_CBC56_DES,
    "privkey1",
)
config.add_target_parameters(snmpEngine, "my-creds", "usr-md5-des", "authPriv")

# Setup transport endpoint and bind it with security settings yielding
# a target name
config.add_transport(
    snmpEngine, udp.DOMAIN_NAME, udp.UdpAsyncioTransport().open_client_mode()
)
config.add_target_address(
    snmpEngine,
    "my-nms",
    udp.DOMAIN_NAME,
    ("127.0.0.1", 162),
    "my-creds",
    tagList="all-my-managers",
)

# Specify what kind of notification should be sent (TRAP or INFORM),
# to what targets (chosen by tag) and what filter should apply to
# the set of targets (selected by tag)
config.add_notification_target(
    snmpEngine, "my-notification", "my-filter", "all-my-managers", "trap"
)

# Allow NOTIFY access to Agent's MIB by this SNMP model (3), securityLevel
# and SecurityName
config.add_context(snmpEngine, "")
config.add_vacm_user(snmpEngine, 3, "usr-md5-des", "authPriv", (), (), (1, 3, 6))

# *** SNMP engine configuration is complete by this line ***

# Create Notification Originator App instance.
ntfOrg = ntforg.NotificationOriginator()

# Build and submit notification message to dispatcher
ntfOrg.send_varbinds(
    snmpEngine,
    # Notification targets
    "my-notification",  # notification targets
    None,
    "",  # contextEngineId, contextName
    # var-binds
    [
        # SNMPv2-SMI::snmpTrapOID.0 = SNMPv2-MIB::coldStart
        (
            (1, 3, 6, 1, 6, 3, 1, 1, 4, 1, 0),
            v2c.ObjectIdentifier((1, 3, 6, 1, 6, 3, 1, 1, 5, 1)),
        ),
        # additional var-binds: ( (oid, value), ... )
        ((1, 3, 6, 1, 2, 1, 1, 5, 0), v2c.OctetString("Notificator Example")),
    ],
)

print("Notification is scheduled to be sent")

# Run I/O dispatcher which would send pending message and process response
snmpEngine.open_dispatcher()



#!/usr/bin/env python3
"""
Script to switch Python versions for development.
"""

import argparse
import os
import shutil
import subprocess
import sys
from pathlib import Path


def get_python_path(version):
    """Get path to Python executable from pyenv."""
    try:
        return (
            subprocess.check_output(["pyenv", "which", f"python{version}"])
            .decode()
            .strip()
        )
    except subprocess.CalledProcessError as e:
        print(f"Error finding Python {version} with pyenv: {e}", file=sys.stderr)
        sys.exit(1)


def main():
    """Main function to switch Python versions."""
    parser = argparse.ArgumentParser(
        description="Switch Python version for development."
    )
    parser.add_argument(
        "python_version", nargs="?", help="Python version to switch to (e.g., 3.12)"
    )

    args = parser.parse_args()

    python_version_file = Path(".python-version")
    current_version = None
    if python_version_file.exists():
        with open(python_version_file) as f:
            current_version = f.read().strip()

    python_version = args.python_version or current_version
    if not python_version:
        print(
            "Error: No Python version specified and .python-version not found.",
            file=sys.stderr,
        )
        sys.exit(1)

    print(f"Switching to Python {python_version}")

    need_switch = current_version != python_version if current_version else True
    if need_switch:
        print(
            "\n[VS Code WARNING] If you are using Visual Studio Code:\n"
            "  - Please update the default Python interpreter (Ctrl+Shift+P → 'Python: Select Interpreter') to match the new version.\n"
            "  - Also open a new terminal after switching Python versions.\n"
        )

        subprocess.run(["pyenv", "local", python_version], check=True)

        venv_path = Path(".venv")
        if venv_path.exists():
            print("Removing existing virtual environment...")
            shutil.rmtree(venv_path)

        python_path = get_python_path(python_version)
        subprocess.run(["uv", "venv", f"--python={python_path}"], check=True)

        print("Installing dependencies...")
        subprocess.run(["uv", "pip", "install", "-e", ".[dev]"], check=True)
    else:
        print(f"Already using Python {python_version}")
        subprocess.run(["uv", "sync", "--extra", "dev"], check=True)

    print(f"Successfully switched to Python {python_version}")
    print("To activate this environment manually, run:")
    print("  source .venv/bin/activate  # On macOS/Linux")
    print("  .venv\\Scripts\\activate    # On Windows")


if __name__ == "__main__":
    main()



"""
SNMPv1
++++++

Send SNMP GET request using the following options:

  * with SNMPv1, community 'public'
  * over IPv4/UDP
  * to an Agent at demo.pysnmp.com:161
  * for an instance of SNMPv2-MIB::sysDescr.0 MIB object
  * Based on asyncio I/O framework

Functionally similar to:

| $ snmpgetnext -v1 -c public demo.pysnmp.com SNMPv2-MIB::sysDescr.0

"""  #
import pytest
from pysnmp.hlapi.v3arch.asyncio import *
from pysnmp.smi.rfc1902 import ObjectIdentity, ObjectType
from tests.agent_context import AGENT_PORT, AgentContextManager


@pytest.mark.asyncio
async def test_v2_next():
    async with AgentContextManager():
        with SnmpEngine() as snmpEngine:
            errorIndication, errorStatus, errorIndex, varBinds = await next_cmd(
                snmpEngine,
                CommunityData("public"),
                await UdpTransportTarget.create(("localhost", AGENT_PORT)),
                ContextData(),
                ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
            )

            assert errorIndication is None
            assert errorStatus == 0
            assert errorIndex == 0
            assert len(varBinds) == 1
            assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysObjectID.0"



"""
SNMPv2c
+++++++

Send SNMP GET request using the following options:

  * with SNMPv2c, community 'public'
  * over IPv4/UDP
  * to an Agent at demo.pysnmp.com:161
  * for two instances of SNMPv2-MIB::sysDescr.0 MIB object,

Functionally similar to:

| $ snmpget -v2c -c public demo.pysnmp.com SNMPv2-MIB::sysDescr.0

"""  #
import asyncio
from pysnmp.hlapi.v3arch.asyncio import *
from pysnmp.smi.rfc1902 import ObjectIdentity, ObjectType


async def run():
    snmpEngine = SnmpEngine()
    errorIndication, errorStatus, errorIndex, varBinds = await get_cmd(
        snmpEngine,
        CommunityData("public"),
        await UdpTransportTarget.create(("demo.pysnmp.com", 161)),
        ContextData(),
        ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
    )

    if errorIndication:
        print(errorIndication)
    elif errorStatus:
        print(
            "{} at {}".format(
                errorStatus.prettyPrint(),
                errorIndex and varBinds[int(errorIndex) - 1][0] or "?",
            )
        )
    else:
        for varBind in varBinds:
            print(" = ".join([x.prettyPrint() for x in varBind]))


asyncio.run(run())



"""
Fetch scalar MIB variables (SNMPv1)
+++++++++++++++++++++++++++++++++++

Perform SNMP GET operation with the following options:

* with SNMPv1, community 'public'
* over IPv4/UDP
* to an Agent at demo.pysnmp.com:161
* for OIDs in tuple form

This script performs similar to the following Net-SNMP command:

| $ snmpget -v1 -c public -ObentU demo.pysnmp.com 1.3.6.1.2.1.1.1.0 1.3.6.1.2.1.1.3.0

"""  #
from pysnmp.carrier.asyncio.dispatch import AsyncioDispatcher
from pysnmp.carrier.asyncio.dgram import udp, udp6
from pyasn1.codec.ber import encoder, decoder
from pysnmp.proto import api
from pysnmp.proto.rfc1905 import ResponsePDU

# Protocol version to use
pMod = api.PROTOCOL_MODULES[api.SNMP_VERSION_1]
# pMod = api.PROTOCOL_MODULES[api.SNMP_VERSION_2C]

# Build PDU
reqPDU = pMod.GetRequestPDU()
pMod.apiPDU.set_defaults(reqPDU)
pMod.apiPDU.set_varbinds(
    reqPDU, (("1.3.6.1.2.1.1.1.0", pMod.Null("")), ("1.3.6.1.2.1.1.3.0", pMod.Null("")))
)

# Build message
reqMsg = pMod.Message()
pMod.apiMessage.set_defaults(reqMsg)
pMod.apiMessage.set_community(reqMsg, "public")
pMod.apiMessage.set_pdu(reqMsg, reqPDU)


# noinspection PyUnusedLocal,PyUnusedLocal
def cbRecvFun(
    transportDispatcher, transportDomain, transportAddress, wholeMsg, reqPDU=reqPDU
):
    while wholeMsg:
        rspMsg, wholeMsg = decoder.decode(wholeMsg, asn1Spec=pMod.Message())
        rspPDU: ResponsePDU = pMod.apiMessage.get_pdu(rspMsg)

        # Match response to request
        if pMod.apiPDU.get_request_id(reqPDU) == pMod.apiPDU.get_request_id(rspPDU):
            # Check for SNMP errors reported
            errorStatus = pMod.apiPDU.get_error_status(rspPDU)
            if errorStatus:
                print(errorStatus.prettyPrint())

            else:
                for oid, val in pMod.apiPDU.get_varbinds(rspPDU):
                    print(f"{oid.prettyPrint()} = {val.prettyPrint()}")

            transportDispatcher.job_finished(1)

    return wholeMsg


transportDispatcher = AsyncioDispatcher()

transportDispatcher.register_recv_callback(cbRecvFun)

# UDP/IPv4
transportDispatcher.register_transport(
    udp.DOMAIN_NAME, udp.UdpAsyncioTransport().open_client_mode()
)

# Pass message to dispatcher
transportDispatcher.send_message(
    encoder.encode(reqMsg), udp.DOMAIN_NAME, ("demo.pysnmp.com", 161)
)
transportDispatcher.job_started(1)

## UDP/IPv6 (second copy of the same PDU will be sent)
# transportDispatcher.register_transport(
#    udp6.domainName, udp6.Udp6AsyncioTransport().open_client_mode()
# )

# Pass message to dispatcher
# transportDispatcher.send_message(
#    encoder.encode(reqMsg), udp6.domainName, ('::1', 161)
# )
# transportDispatcher.job_started(1)

# Dispatcher will finish as job#1 counter reaches zero
transportDispatcher.run_dispatcher(3)

transportDispatcher.close_dispatcher()



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#

__all__ = ["CommunityData"]


class CommunityData:
    """Creates SNMP v1/v2c configuration entry.

    This object can be used by
    :py:class:`~pysnmp.hlapi.v1arch.asyncio.AsyncCommandGenerator` or
    :py:class:`~pysnmp.hlapi.v1arch.asyncio.AsyncNotificationOriginator`
    and their derivatives for conveying SNMP v1/v2c configuration.

    Parameters
    ----------
    communityName: py:class:`str`
        SNMP v1/v2c community string.
    mpModel: py:class:`int`
        SNMP version - 0 for SNMPv1 and 1 for SNMPv2c.

    Examples
    --------
    >>> from pysnmp.hlapi.v1arch import CommunityData
    >>> CommunityData('public')
    CommunityData(communityName=<COMMUNITY>, mpModel=1)
    """

    def __init__(self, communityName, mpModel=1):
        """Create a community data instance."""
        self.mpModel = mpModel
        self.communityName = communityName

    def __hash__(self):
        """Return a hash value of the object."""
        return hash((self.communityName, self.mpModel))

    def __repr__(self):
        """Return a string representation of the object."""
        return f"{self.__class__.__name__}(communityName={self.communityName!r}, mpModel={self.mpModel!r})"
